#!/usr/bin/env python
# 
# 2009-01-23
#
# This report will generate a list of bugs that are tagged needs-packaging and
# have more than 0 me-toos (remember the reporter is not counted) or more than
# 5 subscribers


from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from launchpadlib.credentials import Credentials

import os, sys

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir,0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, EDGE_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, EDGE_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile,"w",0600)) 
    
def save_entry(entry):
    try:
        entry.lp_save()
    except HTTPError, error:
        print error.content
    else:
        print "Did not get expected HTTPError!"

try:
    ubuntu = launchpad.distributions['ubuntu']
except:
    print "The Ubuntu distribution is not in Launchpad?"

np_tasks = ubuntu.searchTasks(tags='needs-packaging')

# write to this file
datafile = open('needs-packaging-popularity.html', 'w')

templatefile = open('template.html', 'r')

for line in templatefile:
    if line.strip() == "<!-- *** Title Space *** -->":
        datafile.write("Needs Packaging Popularity\n")
    elif line.strip() == "<!-- *** Header Space *** -->":
        datafile.write("Needs Packaging Popularity\n")
    elif line.strip() == "<!-- *** Paragraph Space *** -->":
        datafile.write("These bug reports are tagged needs-packaging and have a fair number of people interested in them.  All of the column headers are sortable give them a click!")
    elif line.strip() == "<!-- *** Table Header Space *** -->":
        table_header = "<th></th>"  # bug number in this column
        table_header += "<th>Summary</th>"
        table_header += "<th>In</th>"
        table_header += "<th>Importance</th>"
        table_header += "<th>Status</th>"
        table_header += "<th>Users Affected</th>"
        table_header += "<th>Subscriber Count</th>"
        table_header += "<th>Duplicate Count</th>"
        table_header += "\n"
        datafile.write(table_header)
    elif line.strip() == "<!-- *** Table Body Space *** -->":
        for task in np_tasks:
            # bug reporter is not counted in the users_affected_count so use 0 as the minimum
            try:
                if len(task.bug.subscriptions) > 5 or task.bug.users_affected_count > 0:
                    table_row = "<tr>"
                    table_row += "<td>%s</td>" % task.bug.id
                    table_row += "<td><a href='http://launchpad.net/bugs/%s'>%s</a></td>" % (task.bug.id, task.bug.title.encode('utf-8'))
                    table_row += "<td>%s</td>" % task.bug_target_display_name.encode('utf-8')
                    table_row += "<td>%s</td>" % task.importance.encode('utf-8')
                    table_row += "<td>%s</td>" % task.status.encode('utf-8')
                    table_row += "<td>%s</td>" % task.bug.users_affected_count
                    table_row += "<td>%s</td>" % len(task.bug.subscriptions)
                    table_row += "<td>%s</td>" % len(task.bug.duplicates)
                    table_row += "</tr>"
                    table_row += "\n"
                    datafile.write(table_row)
            except HTTPError, error:
                print >> sys.stderr.write('There was an error with LP: %s: %s'
% (task.title.split(' ')[1].replace('#',''), error.response['status']))
    else:
        datafile.write(line)
datafile.close()
