#!/usr/bin/env python
#
# Copyright 2009 Canonical, Ltd
# Author: Brian Murray <brian@ubuntu.com>
# License: GPLv2
# 
# 2009-01-28 
# This will create a report that finds all the bugs for a particular package
# containing bugs with more than 2 duplicates.  It replaces functionality that
# was provided by bugnumbers.

from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT, STAGING_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from launchpadlib.credentials import Credentials

import os, sys

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir,0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, EDGE_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, EDGE_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile,"w",0600)) 
    
def save_entry(entry):
    try:
        entry.lp_save()
    except HTTPError, error:
        print error.content
    else:
        print "Did not get expected HTTPError!"

package = sys.argv[1]

# write to this file
datafile = open('%s.html' % package, 'w')

# commonly used things
ubuntu = launchpad.distributions['ubuntu']

target_package = ubuntu.getSourcePackage(name='%s' % package)

# sort by number of duplicates to reduce quantity of bugs parsed
tasks = target_package.searchTasks(order_by='-number_of_duplicates')

templatefile = open('template.html', 'r')

for line in templatefile:
    if line.strip() == "<!-- *** Title Space *** -->":
        datafile.write("Bug tasks for %s with more than 2 duplicates\n" % package)
    elif line.strip() == "<!-- *** Header Space *** -->":
        datafile.write("Bug tasks for %s with more than 2 duplicates\n" % package)
    elif line.strip() == "<!-- *** Paragraph Space *** -->":
        datafile.write("All of the column headers are sortable give them a click!")
    elif line.strip() == "<!-- *** Table Header Space *** -->":
        table_header = "<th></th>"  # bug number in this column
        table_header += "<th>Summary</th>"
        table_header += "<th>In</th>"
        table_header += "<th>Importance</th>"
        table_header += "<th>Status</th>"
        table_header += "<th>Duplicate Count</th>"
        table_header += "\n"
        datafile.write(table_header)
    elif line.strip() == "<!-- *** Table Body Space *** -->":
        for task in tasks:
            try:
                dupe_count = len(task.bug.duplicates)
                if dupe_count >= 2:    
                    table_row = "<tr>"
                    table_row += "<td>%s</td>" % task.bug.id
                    table_row += "<td><a href='http://launchpad.net/bugs/%s'>%s</a></td>" % (task.bug.id, task.bug.title.encode('utf-8'))
                    table_row += "<td>%s</td>" % task.bug_target_display_name.encode('utf-8')
                    table_row += "<td>%s</td>" % task.importance.encode('utf-8')
                    table_row += "<td>%s</td>" % task.status.encode('utf-8')
                    table_row += "<td>%s</td>" % ( dupe_count )
                    table_row += "</tr>"
                    table_row += "\n"
                    datafile.write(table_row)
                elif dupe_count < 2:
                    break
            except HTTPError, error:
                print "There was an error with LP: #%s: %s" % (task.title.split(' ')[1].replace('#',''), error.content)
    else:
        datafile.write(line)
datafile.close()
