#!/usr/bin/env python
##########################################################################
# automatically generate html page which lists bugs tagged "qa-hardy-list"
##########################################################################
import os
import sys
import commands
import re
import urllib
import getopt
import datetime
import itertools
import codecs

from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT, STAGING_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from launchpadlib.credentials import Credentials

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir,0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, EDGE_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, EDGE_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile,"w",0600))

def usage():
    print 'python ./support-team-list.py'

TIMESTAMP = commands.getoutput('date -u +"%b %e %G %k:%M %Z"')
TITLE = 'Canonical Support Team Buglist'
HOME = "/home/brian"
PUBLIC_HTML = HOME + "/public_html"
HTML_FILE = PUBLIC_HTML + '/support-team.html'
SUPPORT_TEAM_ARCHIVE = PUBLIC_HTML + "/support-team-archive"
SUPPORT_TEAM_ARCHIVE_URL = "https://chinstrap.canonical.com/~brian/support-team-archive"


URL = 'https://bugs.launchpad.net/~canonical-support'

ubuntu = launchpad.distributions['ubuntu']
team = launchpad.people['canonical-support']
buglist = ubuntu.searchTasks(bug_subscriber=team)

active_series = [series for series in ubuntu.series if series.active]

series_tasks = []
for series in active_series:
    # would be better to use if len(series.searchTasks(..)) > 0 when it becomes
    # available
    for task in series.searchTasks(bug_subscriber=team,
                                   omit_targeted=False):
        series_tasks.append(task)

support_list = itertools.chain(buglist, series_tasks)
#print support_list
html = """
<html>
<head>
    <title>%s - %s</title>""" % (TITLE, TIMESTAMP)
html += """
</head>
<p>
<table class="sortable listing" id="trackers" border=1>
<thead><tr>
    <th align=center>Bug</defanghtml_hd>
    <th align=center>Summary</th>
    <th align=center>Reported By</th>
    <th align=center>Source Package(s)</th>
    <th align=center>Importance</th>
    <th align=center>Status</th>
    <th align=center>Assignee</th>
    <th align=center>Days Open</th>
</tr></thead>
"""

GREEN = '#c4ffc4'
YELLOW = '#fffec4'

today = str(datetime.date.today())
today_split = today.split('-')
support_titles = ("P1: ", "P2: ", "P3: ")

processed = []
for bug_task in support_list:
    try:
        URL = 'https://bugs.launchpad.net/bugs/%s' % bug_task.bug.id
        # only show the private bug reports because they are the ones that are
        # escalated
        if not bug_task.bug.private or bug_task.bug.id in processed:
            continue
        # if the title does not start with "P[123]: " it is not an escalated
        # support team bug
        # N.B. note the space after the colon
        elif not bug_task.bug.title.startswith(support_titles):
            continue
        processed.append(bug_task.bug.id)
        count = 0
        html_first_row = ""
        html_row_end = ''
        ref = re.search("REF:\s*(\S*)",bug_task.bug.description)
        if ref != None:
                REF = ref.group(1)
        else:
                REF = ""
        for task in bug_task.bug.bug_tasks:
            try:
                if task.bug.id == 288774 and task.status == "New":
                    continue
                if (task.status == "Invalid" or task.status == "Won\'t Fix" or
                    task.status == "Fix Released" or "Ubuntu" not in
task.bug_target_display_name):
                    continue
                if task.status == 'New':
                    color = YELLOW
                else:
                    color = ''

                html_row = ''

                if count > 0:
                    html_row += "<tr>\n"

                html_row += "\t<td>%s</td>\n" % (task.bug_target_display_name)
                html_row += "\t<td>%s</td>\n" % task.importance
                html_row += "\t<td bgcolor=\"%s\">%s</td>\n" % (color, task.status)
                assignee = ""

                if task.assignee_link:
                    assignee = task.assignee.display_name

                html_row += "\t<td>%s</td>\n" % assignee
                date = re.match("(\d\d\d\d-\d\d-\d\d).*", str(task.bug.date_created))
                date = date.group(1)
                date_split = date.split('-')
                days = datetime.date(int(today_split[0]), int(today_split[1]), int(today_split[2])) - datetime.date(int(date_split[0]), int(date_split[1]), int(date_split[2]))
                days = str(days).split(' ')

                if days[0] == "0:00:00":
                    days[0] = 0

                days_color = ''
                if int(days[0]) > 15:
                    days_color = YELLOW

                html_row += "\t<td bgcolor=\"%s\"><b>%s</b> days (reported: %s)</td>\n" % (days_color, days[0], date)
                if count == 0:
                    html_first_row = html_row
                else:
                    html_row += "</tr>\n"
                    html_row_end += html_row
                count += 1
                break
            except:
                print '    Sorry:', sys.exc_type, ':', sys.exc_value
                print '    Omitting package from the list'

        html_row_start = "<tr>\n"
        html_row_start += "\t<td class=\"first\" rowspan=\"%d\"><a href=\"%s\">%d</a></td>\n" % (count, URL, bug_task.bug.id)
        if REF:
            html_row_start += '\t<td rowspan="%d"><a href="%s">%s</a></td>\n'\
                % (count, "https://emea.salesforce.com/" + REF, bug_task.bug.title)
        else:
            html_row_start += "\t<td rowspan=\"%d\">%s</td>\n" % (count,
                bug_task.bug.title.encode)
        reporter = bug_task.bug.owner.display_name
        html_row_start += "\t<td rowspan=\"%d\">%s</td>\n" % (count, reporter)
        html_row_start += html_first_row
        html_row_start += html_row_end
        html += html_row_start

    except:
        print '    Sorry:', sys.exc_type, ':', sys.exc_value
        print '    Omitting bug %s from the list' % bug_task.bug.id

html += "</table></p></body></html>\n"

templatefile = open('/home/brian/source-trees/ubuntu-qa-tools/main/bug-report-framework/mult-tables-template.html', 'r')
datafile = codecs.open(HTML_FILE, mode='w+', encoding='utf-8')
for line in templatefile:
    if line.strip() == "<!-- *** Title Space *** -->":
        datafile.write("Canonical Support Team Bug List\n")
    elif line.strip() == "<!-- *** Header Space *** -->":
        datafile.write("<a href=\"%s\">Canonical Support Team Bug List</a> - %s\n" % ("https://bugs.launchpad.net/~canonical-support", TIMESTAMP))
    elif line.strip() == "<!-- *** Table Space *** -->":
        datafile.write(html)
    else:
        datafile.write(line)
templatefile.close()
datafile.close()
