#!/usr/bin/python
#
# Copyright 2008  Canonical, Ltd
# Author: Brian Murray <brian@ubuntu.com>
# License: GPLv2
#
# using py-lp-b get bug statistics for a package

import sys
import time
import os

from db import *
from storm.locals import *
from launchpadbugs.connector import ConnectBugList
from launchpadbugs.basebuglistfilter import URLBugListFilter

now = int(time.time())
sourcepackage = sys.argv[1]

url = "https://bugs.launchpad.net/ubuntu/+source/%s/+bugs" % sourcepackage

database = create_database("postgres:bug_stats")
store = Store(database)

package = store.find(Package, Package.name == unicode(sourcepackage)).one()

statuses = { 'new':'New', 'incomplete':'Incomplete', 'incomplete_w':'INCOMPLETE_WITH_RESPONSE', 'incomplete_wo':'INCOMPLETE_WITHOUT_RESPONSE', 'confirmed':'Confirmed', 'triaged':'Triaged', 'inprogress':'In Progress', 'fixcommitted':'Fix Committed', 'fixreleased':'Fix Released', 'invalid':'Invalid', '''wontfix''':'''Won't Fix''' }
importances = [ 'Undecided', 'Wishlist', 'Low', 'Medium', 'High', 'Critical' ]
tags = [ 'apport-crash', 'apport-bug', 'apport-package' ]
filters = [ 'has-patch' ]

BugList = ConnectBugList("text")
BugList.authentication = "bug-stats.cookie"
bug_filter = URLBugListFilter()
bug_filter.add_option("status", ("New", "Incomplete", "Confirmed", "Triaged", "In Progress", "Fix Committed",  "Invalid", "Won't Fix", "Fix Released"))
all_bugs =  BugList(bug_filter(url))

#statistic = Statistic()
statistic = Statistic( pkgid = None, time = None, total = None, new = None, incomplete = None, confirmed = None, triaged = None, inprogress = None, fixcommitted = None, fixreleased = None, invalid = None, wontfix = None, total_undecided = None, total_wishlist = None, total_low = None, total_medium = None, total_high = None, total_critical = None, incomplete_w = None, incomplete_wo = None, open_undecided = None, open_wishlist = None, open_low = None, open_medium = None, open_high = None, open_critical = None, total_apportcrash = None, open_apportcrash = None, total_apportbug = None, open_apportbug = None, total_apportpackage = None, open_apportpackage = None, total_haspatch = None, open_haspatch = None, duplicates = None )

statistic.pkgid = package.id
statistic.time = now
statistic.total = len(all_bugs)

# total bugs isn't really the total because duplicates aren't included
#  by default in Launchpad searches
# recording the quantity of duplicates, which is most intereting to me
bug_filter.add_option("duplicates", "off")
all_bugs_dupes = BugList(bug_filter(url))
statistic.duplicates = (len(all_bugs_dupes) - len(all_bugs))

# get quantity for every status
for status in statuses.keys():
    bug_filter = URLBugListFilter()
    bug_filter.add_option('status', ( statuses[status] ))
    buglist = BugList(bug_filter(url))
    value = len(buglist)
    setattr(statistic, status, value)
    store.add(statistic)
#    store.flush()
#store.commit()
for importance in importances:
    bug_filter = URLBugListFilter()
    # this includes closed bugs
    bug_filter.add_option("status", ("New", "Incomplete", "Confirmed", "Triaged", "In Progress", "Fix Committed",  "Invalid", "Won't Fix", "Fix Released"))
    bug_filter.add_option("importance", (importance))
    buglist = BugList(bug_filter(url))
    value = len(buglist)
    column = 'total_' + importance.lower()
    setattr(statistic, column, value)
#    store.add(statistic)
#    store.flush()
    bug_filter = URLBugListFilter()
    # only open bugs
    bug_filter.add_option("status", ("New", "Incomplete", "Confirmed", "Triaged", "In Progress", "Fix Committed"))
    bug_filter.add_option("importance", (importance))
    buglist = BugList(bug_filter(url))
    value = len(buglist)
    column = 'open_' + importance.lower()
    setattr(statistic, column, value)
    store.add(statistic)
#    store.flush()
for tag in tags:
    bug_filter = URLBugListFilter()
    # this includes closed bugs
    bug_filter.add_option("status", ("New", "Incomplete", "Confirmed", "Triaged", "In Progress", "Fix Committed",  "Invalid", "Won't Fix", "Fix Released"))
    bug_filter.add_option("tag", (tag))
    buglist = BugList(bug_filter(url))
    value = len(buglist)
    column = 'total_' + tag.replace('-','')
    setattr(statistic, column, value)
    store.add(statistic)
#    store.flush()
    bug_filter = URLBugListFilter()
    # only open bugs
    bug_filter.add_option("status", ("New", "Incomplete", "Confirmed", "Triaged", "In Progress", "Fix Committed"))
    bug_filter.add_option("tag", (tag))
    buglist = BugList(bug_filter(url))
    value = len(buglist)
    column = 'open_' + tag.replace('-','')
    # use setattr for dynamic column naming
    setattr(statistic, column, value)
    store.add(statistic)
#    store.flush()
for filter in filters:
    bug_filter = URLBugListFilter()
    # this includes closed bugs
    bug_filter.add_option("status", ("New", "Incomplete", "Confirmed", "Triaged", "In Progress", "Fix Committed",  "Invalid", "Won't Fix", "Fix Released"))
    bug_filter.add_option(filter, ("on",))
    buglist = BugList(bug_filter(url))
    value = len(buglist)
    column = 'total_' + filter.replace('-','')
    # use setattr for dynamic column naming
    setattr(statistic, column, value)
    store.add(statistic)
#    store.flush()
    bug_filter = URLBugListFilter()
    # this includes open bugs
    bug_filter.add_option("status", ("New", "Incomplete", "Confirmed", "Triaged", "In Progress", "Fix Committed"))
    bug_filter.add_option(filter, ("on",))
    buglist = BugList(bug_filter(url))
    value = len(buglist)
    column = 'open_' + filter.replace('-','')
    #print '%s is %s' % (column, value)
    # use setattr for dynamic column naming
    setattr(statistic, column, value)
    store.add(statistic)
#    store.flush()
store.commit()
store.close()
