#!/usr/bin/env python
# Copyright 2007-2008  Canonical, Ltd
# Author: Kees Cook <kees@ubuntu.com>
# Author: Brian Murray <brian@ubuntu.com>
# License: GPLv2
import sys

data = 'stats-bugs-with-patches.csv'

current = open('%s' % data).read().splitlines().pop().split(",")[0]

# this will just exit without passing anything to gnuplot if the latest stat check resulted in a launchpad connection timeout
if current.startswith('#'):
    sys.exit(1)
else:
    now = int(current)

print '''
set xdata time
set timefmt "%s"
set xtics rotate by 90
set xlabel "Date/Time" 0,-1
set ylabel "Number of Bugs"
set autoscale
set yrange [0:*]
set offsets 0, 0, 1, 1
set term svg
set grid
set key horizontal outside
'''
# set key horizontal - my favorite
# FAIL - set arrow from 1207676872, 0 to 1207676872, 300 
# colours can be found by using gnuplot's test image for the correct term
# for svgs -
# 1 is red, 3, is blue, 5 is green, 11 is yellowish
statinfo = {
#    'total': { 'title':'Total', 'column':2 },
#    'outstanding': { 'title':'Outstanding', 'column':3, 'colour':1 },
    'review-queue': { 'title':'Review Queue', 'column':4, 'colour':11 },
    'main-sponsors': { 'title':'Main Sponsors', 'column':5, 'colour':3 },
    'universe-sponsors': { 'title':'Universe Sponsors', 'column':6, 'colour':5 },
    'sponsors-queue': { 'title':'Sponsors Queue', 'column':8, 'colour':1 }, # a different color might be good here
    }

# using a list so the statinfo dictionary will be accessed in this order
# useful for having the legend/key sorted
categories = [ 'review-queue', 'sponsors-queue', 'main-sponsors', 'universe-sponsors' ]

days = {
    'fullyear': { 'step': 360, 'span': 3600*24*365, 'format': '%b %Y' },
    'halfyear': { 'step': 180, 'span': 3600*24*180, 'format': '%b %Y' },
    '3month': { 'step': 90, 'span': 3600*24*90, 'format': '%d %b' },
    'month': { 'step': 30, 'span': 3600*24*30, 'format': '%d %b' },
    'week': { 'step': 6, 'span': 3600*24*7, 'format': '%d %b' },
    '1day': { 'step': 1, 'span': 3600*24, 'format': '%m/%d %H:%M' },
    }

for day in days.keys():
    dfile  = data
    xmin = now - days[day]['span']
    dstep  = days[day]['step']

    print '''set format x "%s"''' % (days[day]['format'])
    print '''set output "plots/patch-queue-%s.svg"''' \
            % (day)
    print '''plot ["%d":]''' % (xmin),

    plots = []
    # for every stat plot them together in one graph
    for category in categories:
        title  = statinfo[category]['title']
        column = statinfo[category]['column']
        colour = statinfo[category]['colour']
        plots += ['''"%s" using 1:%d every %d title "%s" with linespoints pointtype 7 pointsize 0.6 linecolor %d, "< tail -n 1 %s" using 1:%d notitle with points pointtype 6''' \
                % (dfile, column, dstep, title, colour, dfile, column)]
    print ",".join(plots)
    print ''
