#!/usr/bin/env python
#
# Copyright 2007-2008  Canonical, Ltd
# Author: Kees Cook <kees@ubuntu.com>
# Author: Brian Murray <brian@ubuntu.com>
# License: GPLv3
#
# This plots a regression- tagged series of bug reports for multiple releases
# on a single graph for one particular status e.g. New regression-potential
# tagged bug reports for Intepid - Lucid
#
# This script has vertical lines at arbitrary points in the graph which is
# fantastic however I think is only possible because we know the absolute
# values for the x-point for them

import sys

tag = sys.argv[1]

print '''
set xtics rotate by 90
set xlabel "Days (since archive opening)" 0,-1

set autoscale
set yrange [0:*]
set offsets 0, 0, 1, 1
set term svg
set grid
set key reverse Left outside
set style line 7 linecolor 11
'''

# 2010-04-16 - these could actually be grabbed using launchpadlib too (except
# for RC) 
# Day numbers are for Lucid which is different for previous releases
# but close enough
milestones = { 42:'Alpha1', 77:'Alpha2', 119:'Alpha3', 140:'Beta1',
               161:'Beta2', 175:'RC', 182:'Final' }

# graph 0 is at the bottom and graph 1 is at the top
for day_number in milestones.keys():
    print '''
    set arrow from %s, graph 0 to %s, graph 1 nohead linetype 7
    set label "%s" at %s, graph 1 left rotate offset -1, -2.75 front
    '''\
    % ( day_number, day_number, milestones[day_number], day_number )

# colours can be found by using gnuplot's test image for the correct term
# for svgs -
# 1 is red, 3 is blue, 5 is green, 11 is yellowish
# columns are base 1
statinfo = {
    'open': { 'title':'Open', 'column':2, },
    'new': { 'title':'New', 'column':3, },
    'incomplete': { 'title':'Incomplete', 'column':4, },
    'confirmed': { 'title':'Confirmed', 'column':5, },
    'triaged': { 'title':'Triaged', 'column':6, },
    'inprogress': { 'title':'In Progress', 'column':7 },
    'fixcommitted': { 'title':'Fix Committed', 'column':8 },
# not interested in these as closed stats will continue to grow
#    'fixreleased': { 'title':'Fix Released', 'column':9 },
#    'invalid': { 'title':'Invalid', 'column':10 },
#    'wontfix': { 'title':'''Won't Fix''', 'column':11 },
#    'undecided': { 'title':'Undecided', 'column':12 },
#    'wishlist': { 'title':'Wishlist', 'column':13 },
#    'low': { 'title':'Low', 'column':14 },
#    'medium': { 'title':'Medium', 'column':15 },
#    'high': { 'title':'High', 'column':16 },
#    'critical': { 'title':'Critical', 'column':17 },
    }

# using a list so the statinfo dictionary will be accessed in this order
# useful for having the legend/key sorted
stati = [ 'open', 'new', 'incomplete', 'confirmed', 'triaged', 'inprogress', 'fixcommitted' ]

releases = {
    'intrepid' : 5,   # green
    'jaunty'   : 3,   # blue
    'karmic'   : 1,   # red
    'lucid'    : 10,  # a nice lavendar
    'maverick' : 13,  # some color
}

for status in stati:
    # create one separate image per status
    print '''set output "%s/%s-%s.svg"''' % ( tag, tag, status )
    print '''set ylabel "Quantity"''' #% ( statinfo[status]['title'] )
    print '''set title "%s bugs tagged %s" '''% ( statinfo[status]['title'], tag )
    column = statinfo[status]['column']
    dstep = 1
    title = statinfo[status]['title']
    plots = []
    print '''plot''',
    for release in sorted(releases.keys()):
        # each release has its own separate data file from day0 to the end
        dfile = "%s-%s.data" % ( release, tag )
        colour = releases[release]
        plots += ['''"%s" using 1:%d every %d title "%s" with lines linecolor %d'''\
                                     #.title() capitalizes the first Letter
            % (dfile, column, dstep, release.title(), colour)]
    print ",".join(plots)
