#!/usr/bin/env python
# Copyright 2007-2008  Canonical, Ltd
# Author: Kees Cook <kees@ubuntu.com>
# Author: Brian Murray <brian@ubuntu.com>
# License: GPLv2
#
# This script will plot lines for open and resolved bugs for a release on a
# daily basis
#
# data file looks like
# 
# Date, Opened, Resolved, Open Count, Resolved Count, Total
# 2009-10-29, 1, 1, 0, 1, 1
# 2009-10-30, 16, 0, 16, 1, 17
#
# This script has vertical lines at arbitrary points in the graph which is
# fantastic however I think is only possible because we know the absolute
# values for the x-point for them

# 2010-04-07 - might be useful to specify an end date for older releases

import sys

release = sys.argv[1]

print '''
set xtics rotate by 90
set xlabel "Days (since release opening)" 0,-1

set autoscale
set xrange [0:182]
set yrange [0:*]
set offsets 0, 0, 1, 1
set term svg size 1024, 768
set boxwidth 0.5 relative''' # used to set the default width of boxes in histograms / number of units along the x axis
print '''
set grid
set key reverse Left outside
'''
# a style for vertical lines at arbitrary points
print '''
set style line 7 linecolor 11
'''

# for Lucid - RC doesn't exist in Launchpad but the other day numbers and names
# could be calculated using launchpadlib
milestones = { 42:'Alpha1', 77:'Alpha2', 119:'Alpha3', 140:'Beta1', 161:'Beta2', 175:'RC', 182:'Final' }

# graph 0 is at the bottom and graph 1 is at the top
for milestone in milestones.keys():
    print '''
    set arrow from %s, graph 0 to %s, graph 1 nohead linetype 7
    set label "%s" at %s, graph 1 left rotate offset -1, -2.75 front
    '''\
    % ( milestone, milestone, milestones[milestone], milestone )

# colours can be found by using gnuplot's test image for the correct term
# for svgs -
# 1 is red, 3, is blue, 5 is green, 11 is yellowish
# columns are base 1
statinfo = {
    'open': { 'title':'Open', 'column':3, 'colour':1, 'lt':1 },
    'resolved': { 'title':'Resolved', 'column':4, 'colour':5, 'lt':3 }
#    'new': { 'title':'New', 'column':3, },
#    'incomplete': { 'title':'Incomplete', 'column':4, },
#    'confirmed': { 'title':'Confirmed', 'column':5, },
#    'triaged': { 'title':'Triaged', 'column':6, },
#    'inprogress': { 'title':'In Progress', 'column':7 },
#    'fixcommitted': { 'title':'Fix Committed', 'column':8 },
#    'fixreleased': { 'title':'Fix Released', 'column':9 },
#    'invalid': { 'title':'Invalid', 'column':10 },
#    'wontfix': { 'title':'''Won't Fix''', 'column':11 },
#    'undecided': { 'title':'Undecided', 'column':12 },
#    'wishlist': { 'title':'Wishlist', 'column':13 },
#    'low': { 'title':'Low', 'column':14 },
#    'medium': { 'title':'Medium', 'column':15 },
#    'high': { 'title':'High', 'column':16 },
#    'critical': { 'title':'Critical', 'column':17 },
    }

# using a list so the statinfo dictionary will be accessed in this order
# useful for having the legend/key sorted
#stati = [ 'open', 'new', 'incomplete', 'confirmed', 'triaged', 'inprogress', 'fixcommitted' ]
stati = [ 'open', 'resolved' ]

# make it a histogram (bar graph)
print '''
set style data histograms
set style histogram clustered gap 2
'''
#set style fill pattern border -1 # looks bad
#'''

# 2010-04-02 - using a dstep of 1 they continue to overlap
print '''set output "%s/bug-rate.svg"''' % release
print '''set ylabel "Quantity"''' #% ( statinfo[status]['title'] )
print '''set title "%s Opened and Resolved on a daily basis" ''' % release.title()
plots = []
dfile = "%s-bug-rate.txt" % ( release )
print '''plot''',
for status in stati:
    column = statinfo[status]['column']
    dstep = 1 # changing this to anything else looks hosed
    title = statinfo[status]['title']
    linetype = statinfo[status]['lt']
    plots += ['''"%s" using %d every %d title "%s"''' % ( dfile, column, dstep, statinfo[status]['title'].title() )]
print ",".join(plots)
