#!/usr/bin/env python
######################################################################
# automatically generate html page which lists kernel bugs the
# kernel team might want to look at
######################################################################
import os
import sys
import commands
import re
import urllib
from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from launchpadlib.credentials import Credentials

if len(sys.argv) != 2:
    print 'Usage: %s <bug-list.txt>' % (sys.argv[0])
    sys.exit(1)

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir,0700)

script_name = sys.argv[0].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, EDGE_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, EDGE_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile,"w",0600))

date = commands.getoutput('date +%F')
htmlfile = "./jaunty-buglist-%s.html" % date
date = commands.getoutput('date -u +"%b %e %G %k:%M %Z"')
buglistfile = sys.argv[1]
tags = ["qa-jaunty-desktop", "qa-jaunty-foundations", "qa-jaunty-server", "qa-jaunty-kernel"]
html = "<html>\n"
html += "<head>\n"
html += "\t<title>Jaunty Jackalope Buglist</title>\n"
html += "\t<link defanghtml_rel=\"made\" defanghtml_href=\"mailto:webmaster@ubuntu.com\">\n"
html += "\t<link rel=\"stylesheet\" type=\"text/css\" charset=\"utf-8\" media=\"all\" href=\"http://www.ubuntu.com/htdocs/ubuntuweb/css/common.css\">\n"
html += "\t<link rel=\"stylesheet\" type=\"text/css\" charset=\"utf-8\" media=\"screen\" href=\"http://www.ubuntu.com/htdocs/ubuntuweb/css/screen.css\">\n"
html += "\t<link rel=\"stylesheet\" type=\"text/css\" charset=\"utf-8\" href=\"buglist.css\">\n"
html += "</head>"

html += "<body>\n"
html += "<h3>Jaunty Jackalope Buglist - %s</h3>\n" % (date)
html += '<p>Also visit <a href="http://qa.ubuntu.com/reports/ogasawara/qa-jaunty-buglist.html">http://qa.ubuntu.com/reports/ogasawara/qa-jaunty-buglist.html</a> for an alternate view of this list that allows sorting by column.</p>\n'
html_table = "<p>\n<table border=1>\n"
html_table += "<tbody><tr>\n"
html_table += "\t<th align=center>Bug</defanghtml_hd>\n"
html_table += "\t<th align=center>Source Package</th>\n"
html_table += "\t<th align=center>Importance</th>\n"
html_table += "\t<th align=center>Status</th>\n"
html_table += "\t<th align=center>Summary</th>\n"
html_table += "\t<th align=center>Assignee</th>\n"
html_table += "\t<th align=center>Comments</th>\n"
html_table += "</tr>\n"

teams = {}
buglist = {}

for tag in tags:
    url = "https://launchpad.net/ubuntu/+bugs-text?field.tag=%s" % tag
    if tag is "qa-jaunty-desktop":
        team = "Desktop"
    elif tag == "qa-jaunty-foundations":
        team = "Foundations"
    elif tag == "qa-jaunty-server":
        team = "Server"
    elif tag == "qa-jaunty-kernel":
        team = "Kernel"
    content = urllib.urlopen(url)
    for line in content:
        line = line.strip("\n")
        line = int(line)
        if line not in buglist.keys():
            buglist[line] = {"comment": "",
                            "team": team}

LIST = open(buglistfile, "r")
for line in LIST:
    if re.match('#', line):
        continue
    arg = line.split(',')
    num = int(arg[0])
    comment = arg[1]
    team = arg[2].strip()
    if team == "":
        team = "Other"
    if num not in buglist.keys():
        buglist[num] = {"comment": comment,
                        "team": team}
LIST.close()

quirk = 83166
for num in sorted(buglist.keys()):
    try:
        comment = buglist[num]["comment"]
        team = buglist[num]["team"]
        URL = "https://bugs.launchpad.net/bugs/%d" % num
        if num == quirk:
            URL = "https://bugs.launchpad.net/ubuntu/+source/ubiquity/+bug/83166"
        bug = launchpad.bugs[num]
        length = 0
        html_row = ""
        for task in bug.bug_tasks:
            if "Ubuntu" not in task.bug_target_name and team != "Community":
                continue
            if task.status.encode('utf-8') == 'Fix Released' or task.status.encode('utf-8') == 'Fix Committed':
                color="#c4ffc4"
            elif task.status.encode('utf-8') == 'In Progress':
                color="#fffec4"
            elif task.status.encode('utf-8') == 'Invalid' or task.status.encode('utf-8') == "Won\'t Fix":
                continue
            else:
                color=""
            length = length + 1
            assignee = ""
            if task.assignee:
                assignee = task.assignee.display_name.encode('utf-8')
            html_row += "\t<td>%s</td>\n" % task.bug_target_display_name.encode('utf-8')
            html_row += "\t<td>%s</td>\n" % task.importance.encode('utf-8')
            html_row += "\t<td bgcolor=\"%s\">%s</td>\n" % (color, task.status.encode('utf-8'))
            html_row += "\t<td>%s</td>\n" % task.title.encode('utf-8')
            html_row += "\t<td>%s</td>\n" % assignee
            html_row += "\t<td>%s</td>\n" % comment
            html_row += "</tr>\n"
        html_temp = "<tr>\n\t<td class=\"first\" rowspan=\"%d\"><a href=\"%s\">%s</a></td>\n" % (length, URL, num)
        html_row = html_temp + html_row
        if team not in teams.keys():
            teams[team] = ""
        teams[team] += str(html_row)
    except AttributeError:
        print AttributeError

for key in sorted(teams.keys()):
    html += "<h3>%s</h3>\n" % key
    html += html_table
    html += teams[key]
    html += "</tbody></table>"
html += "<p></body></html>\n"
HTML = open(htmlfile, 'w')
HTML.write(html)
HTML.close()
