#!/usr/bin/env python
######################################################################
#
# New Header: 21 January 2010
# Modified by Jeremy Foshee (JFo)
#
#This is a generic HTML manipulation script.
# It takes input and, based on what the importance/status is, colorizes and sorts the bits into their respective piles
#
# ChangeLog:
# Date                    Description                 Lines affected
#
#
#Old Header: pre 21 January 2010  --- may remove section below this later on based on convo with Leann.
#
# automatically generate a html page which lists kernel bugs the
# kernel team might want to look at:
#
# http://qa.ubuntu.com/reports/ogasawara/kernel-buglist.html
#
# this script requires that you pass in a separate file containing
# the list of bugs.  The list must be in the following format:
#
# <bug number>,<comment>,<ommit>,
#
# for example:
# 112233,kernel config change,,
# 445566,patch upstream,,
# 0,end of the current week's bug list,,
# 778899,bitesize patch,,
# 92953,test bug,ommit,
#
# The resulting display will be sorted by Importance and then
# subsorted by Status.  Bugs which come after the line
# "0,end of the current week's bug list,," will automatically fall
# under the Carried Forward section unless the "ommit" string is
# specified in the 3rd field.
######################################################################
import time
import commands
import inspect
import sys, os, re

RED = "#ffa2a2"
GREEN = "#c4ffc4"
YELLOW = "#fffec4"
GREY = "#e0e0e0"

class Status:
    def __init__(self):
        self.triaged = []
        self.confirmed = []
        self.new = []
        self.inprogress = []
        self.fixcommitted = []
        self.fixreleased = []
        self.incomplete = []
        self.invalid = []
        self.wontfix = []

class SortedList:
    def __init__(self):
        self.critical= Status()
        self.high = Status()
        self.medium = Status()
        self.low = Status()
        self.wishlist = Status()
        self.undecided = Status()
        self.unknown = Status()

    def SortImportance(self, html, importance, status, list):
        temp = importance.lower()
        value = getattr(list, temp)
        self.SortStatus(html, status, value)
        return

    def SortStatus(self, html, status, list):
        stat = status.replace(' ', '').lower()
        stat = stat.replace("'", "")
        value = getattr(list, stat)
        value.append(html)
        return

def ColorStatus(status):
    color=""
    if status == "Fix Released" or status == "Fix Committed":
        color = GREEN
    elif status == "In Progress":
        color = YELLOW
    elif status == "Invalid" or status == "Won\'t Fix":
        color = GREY
    return color

def ColorImportance(importance):
    color = ""
    if importance == "Critical" or importance == "High":
        color = RED
    elif importance == "Medium":
        color = YELLOW
    return color

def BuildBugList(list):
    html = ""
    count = 0
    importance_list = ["critical", "high", "medium", "low", "wishlist", "undecided", "unknown"]
    status_list = ["triaged", "confirmed", "new", "inprogress", "fixcommitted", "fixreleased", "incomplete", "invalid", "wontfix"]
    for importance in importance_list:
        value = getattr(list, importance)
        for status in status_list:
            lines = getattr(value, status)
            for line in lines:
                html += line
                count += 1
    return (html, count)

def CreatePieChart(stats):
    piechartfile= "./kernel-piechart-temp.data"
    piechart_data = "0, 0, %s, %s, %s, %s, %s, %s, %s, %s, %s\n" % (stats["New"], stats["Incomplete"], stats["Confirmed"], stats["Triaged"], stats["In Progress"], stats["Fix Committed"], stats["Fix Released"], stats["Invalid"], stats["Won\'t Fix"])

    FILE = open(piechartfile, "w")
    FILE.write(piechart_data)
    FILE.close()
