#!/usr/bin/env python
# 
# 2010-04-22 
# Review the list of expirable bugs and ensure that they really
# meet the criteria for expiration.
# Criteria detailed at https://help.launchpad.net/Bugs/Expiry

from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from launchpadlib.credentials import Credentials

import os, sys, re, urllib
import datetime

def get_bugs(count):
    # pattern used to find bug numbers since +expirable-bugs is
    # not available in the API
    #p = re.compile('<td class="amount">(\d+)</td>')
    p = re.compile('<a href="/ubuntu/\+source/(.*)/\+bug/(\d+)">')
    bug_numbers = {}
    (f_name, h) = urllib.urlretrieve(\
                "https://bugs.launchpad.net/ubuntu/+expirable-bugs?start=%s" % count )
    for l in open(f_name):
        m = p.search(l)
        if m:
            bug_numbers[m.group(2)] = m.group(1)
    return bug_numbers


cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir,0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, EDGE_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, EDGE_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile,"w",0600))

stati = ['Incomplete']
eligible_stati = [ 'Invalid', "Won't Fix", 'Incomplete' ]

today = datetime.date.today()

# there were 12375 bugs eligible for expiration last time I checked
for start in range(0, 301, 75):
    print "Working on range: %s" % start 
    expirable_bugs = get_bugs(start)

    for bug in expirable_bugs.keys():
        bug_number = bug
        source_package = expirable_bugs[bug]
        url = "%subuntu/+source/%s/+bug/%s" % ( launchpad._root_uri, source_package, bug_number )
        bug_task = launchpad.load(url)
        if bug_task.status not in stati:
            print "LP: #%s has status %s" % ( bug_task.bug.id, bug_task.status )
        if bug_task.bug.duplicate_of:
            print "LP: #%s is a duplicate" % bug_task.bug.id
        if bug_task.assignee:
            print "LP: #%s is assigned to %s" % ( bug_task.bug.id, bug_task.assignee.display_name )
        if bug_task.milestone:
            print "LP: #%s has a milestone" % ( bug_task.bug.id )
        if not bug_task.bug.can_expire:
            print "LP: #%s can not expire" % ( bug_task.bug.id )
        last_update = bug_task.bug.date_last_updated.date()
        #if not bug_task.date_last_updated.date() 
        # oh expirable bugs includes ones that'll be eligible sometime so these
        # would be false positives
        #if abs(today - last_update) <= datetime.timedelta(days=60):
        #    print "LP: #%s has a date_last_updated of %s" % ( bug_task.bug.id, last_update )
        # Bugs watched in external trackers are never candidates for expiry.
        if len(bug_task.bug.bug_watches) > 0:
            print "LP: #%s has a bug watch" % ( bug_task.bug.id )
        for task in bug_task.bug.bug_tasks:
            if task.status not in eligible_stati: #== 'Confirmed':
                print "LP: #%s has a non-eligible bug task" % ( bug_task.bug.id )
        # In a fit of paranoia I added the following to ensure it was working
        #else:
        #    print "LP: #%s is good for expiration" % ( bug_task.bug.id )
