#!/usr/bin/env python
#
# Generate the list of kernel bugs on a per release basis that have a known
# specific patch which fixed the bug.  This is determined by the fact that
# a git commit referenced a "LP:<bug#> in the changelog which triggered the
# Janitor to automatically mark the bug Fix Released.  The file
# "fixed-linux-known-patches.data" contains all the linux bugs marked
# Fix Released by the Janitor on a per release basis.

from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from launchpadlib.credentials import Credentials

import sys,os

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir,0700)

script_name = sys.argv[0].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, EDGE_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, EDGE_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile,"w",0600)) 
    

if len(sys.argv) != 2:
    print "Usage: %s <fixed_linux_known_patches.data>" % (sys.argv[0])
    sys.exit(1)

ubuntu = launchpad.distributions["ubuntu"]
collection = ubuntu.getSourcePackage(name="linux")

FILE = sys.argv[1]
TEMPLATE = "/home/ogasawara/ubuntu-qa-tools/bug-report-framework/template.html"

DATAFILE = open(FILE,'r')

for line in DATAFILE:
    kernel_team_data = line.split(",")
    release = kernel_team_data[0].strip()
    release = release.replace(' ','')
    release = release.replace('.','')
    outfile = "%s-fixed-linux-known-patches.html" % (release.lower())
    OUTFILE = open(outfile, 'w')
    TEMPLATEFILE = open(TEMPLATE, 'r')
    print release
    for line in TEMPLATEFILE:
        if line.strip() == "<!-- *** Title Space *** -->":
            OUTFILE.write("Fixed linux Bugs per Release\n")
        elif line.strip() == "<!-- *** Header Space *** -->":
            OUTFILE.write("<img src='http://qa.ubuntu.com/reports/ogasawara/bugdata/fixed-linux-bugs/fixed-kernel-bugs-bars-stacked.png' alt='Fixed Kernel Bugs'>\n")
            OUTFILE.write("<h1>%s</h1>\n" % kernel_team_data[0].strip())
        elif line.strip() == "<!-- *** Paragraph Space *** -->":
            OUTFILE.write("<h2>%d Bugs Fixed with Known Patches</h2>" % len(kernel_team_data[1:]))
        elif line.strip() == "<!-- *** Table Header Space *** -->":
            table_header = "<th></th>"  # bug number in this column
            table_header += "<th>Summary</th>"
            table_header += "<th>In</th>"
            table_header += "<th>Importance</th>"
            table_header += "<th>Status</th>"
            table_header += "<th>Assignee</th>"
            table_header += "\n"
            OUTFILE.write(table_header)
        elif line.strip() == "<!-- *** Table Body Space *** -->":
            for bug in kernel_team_data[1:]:
                try:
                    bug = bug.strip()
                    tasks = collection.searchTasks(search_text=bug)
                    task = launchpad.load("https://api.edge.launchpad.net/beta/ubuntu/+source/linux/+bug/%s" % bug)
                    table_row = "<tr>"
                    table_row += "<td>%s</td>" % task.bug.id
                    table_row += "<td><a href='http://launchpad.net/bugs/%s'>%s</a></td>" % (task.bug.id, task.bug.title.encode('utf-8'))
                    table_row += "<td>%s</td>" % task.bug_target_display_name.encode('utf-8')
                    table_row += "<td>%s</td>" % task.importance.encode('utf-8')
                    table_row += "<td>%s</td>" % task.status.encode('utf-8')
                    assignee = ""
                    if task.assignee:
                        assignee = task.assignee.name
                    table_row += "<td>%s</td>" % assignee
                    table_row += "</tr>"
                    table_row += "\n"
                    OUTFILE.write(table_row)
                except HTTPError, error:
                    print "There was an error with bug %s: %s" % (bug, error.content)
                    continue
        else:
            OUTFILE.write(line)
    OUTFILE.close()
    TEMPLATEFILE.close()
