# A workaround for Jaunty: The launchpadlib version from Jaunty
# does not yet provide the method Launchpad.login_with(). This
# module provides "surrogate" of this method.

import os.path
import stat
import sys
import urlparse
from launchpadlib.credentials import Credentials
from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT

def lp_login(consumer_name, service_root=EDGE_SERVICE_ROOT,
             launchpadlib_dir=None, timeout=None, proxy_info=None):

    if getattr(Launchpad, 'login_with', None) is not None:
        return Launchpad.login_with(
            consumer_name, service_root, launchpadlib_dir, timeout,
            proxy_info)

    # a plain copy of Launchpad.login_with():
    if launchpadlib_dir is None:
        home_dir = os.environ['HOME']
        launchpadlib_dir = os.path.join(home_dir, '.launchpadlib')
    launchpadlib_dir = os.path.expanduser(launchpadlib_dir)
    # Each service root has its own cache and credential dirs.
    scheme, host_name, path, query, fragment = urlparse.urlsplit(
        service_root)
    service_root_dir = os.path.join(launchpadlib_dir, host_name)
    cache_path = os.path.join(service_root_dir, 'cache')
    if not os.path.exists(cache_path):
        os.makedirs(cache_path)
    credentials_path = os.path.join(service_root_dir, 'credentials')
    if not os.path.exists(credentials_path):
        os.makedirs(credentials_path)
    consumer_credentials_path = os.path.join(
        credentials_path, consumer_name)
    if os.path.exists(consumer_credentials_path):
        credentials = Credentials()

        credentials_file = open(consumer_credentials_path, 'r')
        credentials.load(credentials_file)
        credentials_file.close()
        launchpad = Launchpad(
            credentials, service_root=service_root, cache=cache_path,
            timeout=timeout, proxy_info=proxy_info)
    else:
        launchpad = Launchpad.get_token_and_login(
            consumer_name, service_root=service_root, cache=cache_path,
            timeout=timeout, proxy_info=proxy_info)

        credentials_file = open(
            os.path.join(credentials_path, consumer_name), 'w')
        launchpad.credentials.save(credentials_file)
        credentials_file.close()

        os.chmod(
            os.path.join(credentials_path, consumer_name),
            stat.S_IREAD | stat.S_IWRITE)
    return launchpad

