#!/usr/bin/env python
#
# Copyright 2009 Canonical, Ltd
# Author: Brian Murray <brian@ubuntu.com>
# License: GPLv3
# 
# 2009-02-10
# tool to set all needs-packaging bug reports to an importance of wishlist
# it will also normalize the bug title to start off with [needs-packaging]

from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT, STAGING_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from launchpadlib.credentials import Credentials

import os, sys
import re

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir,0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, EDGE_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, EDGE_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile,"w",0600)) 
    
def save_entry(entry):
    try:
        entry.lp_save()
    except HTTPError, error:
        print error.content

message = '''*** This is an automated message ***\n\nThis bug is tagged needs-packaging which identifies it as a request for a new package in Ubuntu.  As a part of the managing needs-packaging bug reports specification, https://wiki.ubuntu.com/QATeam/Specs/NeedsPackagingBugs, all needs-packaging bug reports have Wishlist importance.  Subsequently, I'm setting this bug's status to Wishlist.'''

# commonly used things
ubuntu = launchpad.distributions['ubuntu']

tasks = ubuntu.searchTasks(importance='Undecided',tags='needs-packaging',order_by='-datecreated')

#print 'There are %s needs-packaging bug reports with an Undecided importance' % len(tasks)

for task in tasks:
    if str(task.bug.id) != '260918':
        bug = task.bug
        summary = bug.title
        if not re.search(r'^[\[\(][Nn]eeds?[- ][Pp]ackaging[\]\)] ?', summary):
            new_summary = '[needs-packaging] ' + summary
            bug.title = new_summary
            save_entry(bug)
        message_subject = "Re: %s" % summary
        try:
            bug.newMessage(content=message, subject=message_subject)
            task.importance = "Wishlist"
            save_entry(task)
        except:
            print "There was an error with LP: #%s" % task.bug.id
        print "Modified LP: #%s" % task.bug.id
