# Generate XML for Package Landing Pages - http://status.qa.ubuntu.com
import urllib2
import re
import datetime
import time
import sys
from xml.dom.minidom import Document
# Refer to http://help.launchpad.net/+apidoc
# Replace STAGING_SERVICE_ROOT with EDGE_SERVICE_ROOT as needed
from launchpadlib.uris import LPNET_SERVICE_ROOT
from launchpadlib.launchpad import Launchpad
from launchpadlib.errors import HTTPError
from launchpadlib.credentials import Credentials

import os, sys

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir,0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, LPNET_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, LPNET_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile,"w",0600))

ubuntu = launchpad.distributions["ubuntu"]

# category.xml contains the xml structure for the drop down list of packages
# to choose from
CATEGORY_FILE = "/srv/qa.ubuntu.com/reports/ogasawara/category.xml"

# location where the xml files can be viewed
XML_URL = "http://qa.ubuntu.com/reports/pkg-stats/"

# category sections and the packages they contain
apps = [
        "banshee",
        "brasero",
        "chromium-browser",
        "compiz",
        "cupsys",
        "ekiga",
        "evince",
        "file-roller",
        "firefox",
        "firefox-3.0",
        "firefox-3.5",
        "flashplugin-nonfree",
        "fontconfig",
        "gwibber",
        "hplip",
        "jockey",
        "metacity",
        "mythtv",
        "network-manager",
        "network-manager-applet",
        "openoffice.org",
        "pulseaudio",
        "samba",
        "scim-bridge",
        "system-config-printer",
        "system-config-samba",
        "thunderbird",
        "tracker",
        "ubuntu-meta",
        "usb-creator",
        "vim",
        "virt-manager",
        "vlc",
        "wine",
        "xulrunner-1.9",
]

gnome = [
        "empathy",
        "epiphany-browser",
        "evolution",
        "evolution-exchange",
        "f-spot",
        "gdm",
        "gedit",
        "gksu",
        "glibc",
        "gnome-app-install",
        "gnome-applets",
        "gnome-control-center",
        "gnome-mount",
        "gnome-panel",
        "gnome-power-manager",
        "gnome-screensaver",
        "gnome-system-monitor",
        "gnome-system-tools",
        "gnome-terminal",
        "gnome-utils",
        "gparted",
        "gstreamer0.10",
        "gvfs",
        "nautilus",
        "pidgin",
        "rhythmbox",
        "totem",
]

kde = [ 
        "amarok",
        "k3b",
        "kdebase",
        "kdenetwork",
        "kdepim",
        "knetworkmanager",
        "kubuntu-meta",
]

installer = [
            "debian-installer",
            "ubiquity",
]

pkg_mgmt = [
            "adept",
            "apt",
            "aptitude",
            "dpkg",
            "gnome-app-install",
            "synaptic",
            "update-manager",
            "update-notifier",
]

xorg = [
        "mesa",
        "usplash",
        "xorg",
        "xorg-server",
        "xserver-xorg-video-ati",
        "xserver-xorg-video-intel",
        "xserver-xorg-video-nv",
]

low_level = [
            "acpi-support",
            "alsa-driver",
            "apparmor",
            "grub",
            "grub2",
            "hal",
            "initramfs-tools",
            "linux",
            "linux-meta",
            "linux-restricted-modules",
            "openssh",
            "sysvinit",
            "util-linux",
]

unity = ["bamf",
         #"compiz-plugins-main",
         "dee",
         "libcompizconfig",
         "libunity-misc",
         #"nux",
         "unity",
         "unity-asset-pool",
         "unity-place-applications",
         "unity-place-files",
         "zeitgeist",
         "zeitgeist-extensions",
]

categories = {"Applications": apps,
              "Installer": installer,
              "Package Management": pkg_mgmt,
              "Xorg": xorg,
              "Low Level": low_level,
              "Gnome": gnome,
              "KDE": kde,
              "Unity": unity,
}

def create_file(file, data):
    FILE = open(file, 'w')
    FILE.write(str(data))
    FILE.close()

#def bug_count(collection):
#        try:
#            content = urllib2.urlopen(url).readlines()
#        except:
#            print "Error reading %s" % (url)
#            return ""
#        uniq_list = Set(content)
#        return str(len(uniq_list))

# Gather version data for Package Info section
def get_pkg_info(pkg, name):
    for archive in ubuntu.archives:
        if archive.name == 'primary':
            primary = archive
            break
    sources = primary.getPublishedSources(source_name=name,exact_match=True)
    pkg["version"]["url"] = "https://launchpad.net/ubuntu/+source/%s" % name
    pkg["version"]["balloon"] = "<span>"
    for source in sources:
        if source.status == "Published":
            if pkg["version"]["count"] == "":
                pkg["version"]["count"] = "%s %s" % (source.distro_series.name.capitalize(), source.source_package_version)
            pkg["version"]["balloon"] += "%s in %s version: %s<br />" % (source.distro_series.name.capitalize(), source.pocket, source.source_package_version)
    pkg["version"]["balloon"] += "</span>"
    return

# Gather data for Bug Info - Totals section
def get_totals(pkg, collection):
    content = ""
    count = ""
    for value in pkg.values():
        if not content:
            try:
                # location of file which contains delta information
                content = urllib2.urlopen(value["changes_file"]).readlines()
            except:
                print "Error reading changes file or getting bug_count"
                return

        item = ""
        if value["delta"] == "New":
            item = 2
        elif value["delta"] == "Incomplete":
            item = 3
        elif value["delta"] == "Confirmed":
            item = 4
        elif value["delta"] == "Triaged":
            item = 5
        elif value["delta"] == "Fix Released":
            item = 8

        tasks = collection.searchTasks(status=value["delta"])
        value["count"] = str(tasks._wadl_resource.representation['total_size'])
        week = content[1].strip().split(',')
        if week[item].find("-") > -1:
            value["delta"] = '&#8595;' + week[item].strip("-")
        elif int(week[item]) > 0:
            value["delta"] = '&#8593;' + week[item]
        else:
            value["delta"] = week[item]
    return

def get_hot_bugs(pkg, collection):
    for value in pkg.values():
        if value.has_key("dups"):
            try:
                tasks = collection.searchTasks(order_by="-number_of_duplicates")
                bug = launchpad.load(tasks[0].bug_link)
                dups = launchpad.load(bug.duplicates_collection_link)
                dups = str(dups._wadl_resource.representation['total_size'])
                value["bug"] += str(bug.id)
                value["dups"] = dups
                value["count"] = dups
                value["title"] = bug.title
                value["status"] = tasks[0].status
                value["importance"] = tasks[0].importance
            except:
                print "Error calculating duplicates"
                continue
        else:
            tasks = collection.searchTasks(importance=value["count"])
            value["count"] = str(tasks._wadl_resource.representation['total_size'])
    return

def get_cold_bugs(pkg, collection):
    for value in pkg.values():
        try:
            tasks = collection.searchTasks(status=value["status"], order_by="datecreated")
            bug = launchpad.load(tasks[0].bug_link)
            value["bug"] += str(bug.id)
            date = str(tasks[0].date_created).partition(" ")
#            tnow = datetime.datetime.now()
#            print tnow.strftime("%Y-%m-%d %H:%M:%S%ff%z")
            date = re.match("(\d\d\d\d-\d\d-\d\d).*", str(date[0]))
#           timestamp = datetime.datetime.strptime(str(date[0]),"%Y-%m-%d")
            timestamp = datetime.datetime.strptime(str(date.group(1)),"%Y-%m-%d")
            diff = datetime.datetime.utcnow() - timestamp
            value["count"] = str(diff.days) + " days"
            value["title"] = bug.title
            value["status"] = tasks[0].status
            value["importance"] = tasks[0].importance
        except:
            # An error can happen if there are 0 tasks in the try statement
            print "Error launchpadlib with %s" % (value["url"])
            value["count"] = "n/a"
            continue

def get_upstream(pkg, collection):
    for value in pkg.values():
        try:
           tasks = collection.searchTasks(status_upstream=value["count"])
           value["count"] = str(tasks._wadl_resource.representation['total_size'])
        except HTTPError, e:
            print e.content
    return

def get_has_patches(value, collection):
    try:
        tasks = collection.searchTasks(has_patch=True)
        value["count"] = str(tasks._wadl_resource.representation['total_size'])
    except HTTPError, e:
        print e.content

def get_ubuntu_sru(value, collection):
    try:
        tasks = collection.searchTasks(bug_subscriber=launchpad.people['ubuntu-sru'])
        value["count"] = str(tasks._wadl_resource.representation['total_size'])
    except HTTPError, e:
        value["count"] = "0"
        print e.content
#def get_dev_and_test(pkg):
#    for value in pkg.values():
#        tasks = collection.searchTasks(status_upstream=value["count"], omit_dupes="on")
#        value["count"] = str(tasks._wadl_resource.representation['total_size'])
#        value["count"] = bug_count(value["url"])
    return

def get_tags(pkg, collection):
    for value in pkg.values():
        tasks = collection.searchTasks(tags=value["count"])
        value["count"] = str(tasks._wadl_resource.representation['total_size'])
#        value["count"] = bug_count(value["url"])
    return

def totals_to_xml(dict, totals):
    xml_desc = xml.createElementNS(None, 'description')
    xml_desc.setAttribute("name", dict["description"])
    totals.appendChild(xml_desc)

    xml_metric = xml.createElementNS(None, 'metric')
    xml_metric.setAttribute("url", dict["url"])

    xml_text = xml.createTextNode(dict["count"])
    xml_metric.appendChild(xml_text)
    xml_desc.appendChild(xml_metric)

    xml_delta = xml.createElementNS(None, 'metric')
    xml_delta.setAttribute("url", dict["delta_url"])
    xml_delta.setAttribute("balloon", dict["balloon"])

    xml_text = xml.createTextNode(dict["delta"])
    xml_delta.appendChild(xml_text)
    xml_desc.appendChild(xml_delta)

    return

def to_xml(dict, stats):
    xml_desc = xml.createElementNS(None, 'description')
    xml_desc.setAttribute("name", dict["description"])
    stats.appendChild(xml_desc)

    xml_metric = xml.createElementNS(None, 'metric')
    if dict.has_key("title"):
        html = "<span><b>%s</b></span><br /><br />\n<table>\n" % (dict["title"])
        html += '\t<tr><td><b>Status</b> : </td><td style="padding-left:2em;color:#291e1c;">%s</td></tr>\n' % (dict["status"])
        html += '\t<tr><td><b>Importance</b> : </td><td style="padding-left:2em;color:#291e1c;">%s</td></tr>\n</table>' % (dict["importance"])
        xml_metric.setAttribute("balloon", html)
        xml_metric.setAttribute("url", dict["bug"])
    else:
        xml_metric.setAttribute("url", dict["url"])
        if dict.has_key("balloon"):
            xml_metric.setAttribute("balloon", dict["balloon"])

    xml_text = xml.createTextNode(dict["count"])
    xml_metric.appendChild(xml_text)
    xml_desc.appendChild(xml_metric)

    return

def to_xml_link(dict, stats):
    xml_desc = xml.createElementNS(None, 'description')
    attr = "<a href=%s>%s</a>" % (dict["url"], dict["description"])
    xml_desc.setAttribute("name", attr)
    stats.appendChild(xml_desc)
    return

keys = categories.keys()

xml_doc = Document()
pkg_stats_xml = xml_doc.createElementNS(None, 'pkg-stats')
xml_doc.appendChild(pkg_stats_xml)

for key in sorted(keys):
    print key
    category_xml = xml_doc.createElementNS(None, 'category')
    category_xml.setAttribute("name", key)
    pkg_stats_xml.appendChild(category_xml)
    for name in sorted(categories[key]):
        print name
        category_pkg = xml_doc.createElementNS(None, 'package')
        pkg_name = xml_doc.createTextNode(name)
        category_pkg.appendChild(pkg_name)
        category_xml.appendChild(category_pkg)
        pkg = {}
        pkg_info = {}
        totals = {}
        hot_bugs = {}
        cold_bugs = {}
        upstream = {}
        regression = {}
        tag = {}
        dev_and_test = {}
        additional_refs = {}
        xml = Document()
        pkg[name] = {}
        xml_pkg = xml.createElementNS(None, 'package')
        xml_pkg.setAttribute("name", name)
        xml_pkg.setAttribute("category", key)
        xml_pkg.setAttribute("xml", XML_URL + name + ".xml")
        date = datetime.datetime.utcnow()
        xml_pkg.setAttribute("timestamp", date.strftime("%Y-%m-%d %H:%M UTC"))
        xml.appendChild(xml_pkg)

        pkg[name]["alt_logo"] = name
#        pkg[name]["graph-dev"] = "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/plots/" + name + "-month-fixing.svg"
        pkg[name]["graph-dev"] = "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/plots/" + name + "-month-fixing.png"
        pkg[name]["graph-dev-url"] = "https://bugs.launchpad.net/ubuntu/+source/" + name + "/+bugs?field.searchtext=&orderby=-importance&search=Search&field.status%3Alist=TRIAGED&field.status%3Alist=INPROGRESS&field.status%3Alist=FIXCOMMITTED"
#        pkg[name]["graph-triage"] = "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/plots/" + name + "-month-triaging.svg"
        pkg[name]["graph-triage"] = "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/plots/" + name + "-month-triaging.png"
        pkg[name]["graph-triage-url"] = "https://bugs.launchpad.net/ubuntu/+source/" + name + "/+bugs?field.searchtext=&orderby=-importance&search=Search&field.status%3Alist=NEW&field.status%3Alist=INCOMPLETE_WITH_RESPONSE&field.status%3Alist=INCOMPLETE_WITHOUT_RESPONSE&field.status%3Alist=CONFIRMED&field.status%3Alist=TRIAGED"

        xml_graphs = xml.createElementNS(None, 'graphs')
        xml_pkg.appendChild(xml_graphs)

        xml_graph = xml.createElementNS(None, 'graph')
        xml_graph.setAttribute("name", "triaged")
        xml_graphs.appendChild(xml_graph)

        xml_url = xml.createElementNS(None, "url")
        xml_text = xml.createTextNode(pkg[name]["graph-triage-url"])
        xml_url.appendChild(xml_text)
        xml_graph.appendChild(xml_url)

        xml_src = xml.createElementNS(None, "src")
        xml_text = xml.createTextNode(pkg[name]["graph-triage"])
        xml_src.appendChild(xml_text)
        xml_graph.appendChild(xml_src)

        xml_graph = xml.createElementNS(None, 'graph')
        xml_graph.setAttribute("name", "devel")
        xml_graphs.appendChild(xml_graph)

        xml_url = xml.createElementNS(None, "url")
        xml_text = xml.createTextNode(pkg[name]["graph-dev-url"])
        xml_url.appendChild(xml_text)
        xml_graph.appendChild(xml_url)

        xml_src = xml.createElementNS(None, "src")
        xml_text = xml.createTextNode(pkg[name]["graph-dev"])
        xml_src.appendChild(xml_text)
        xml_graph.appendChild(xml_src)

        totals[name] = {"new_bugs": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.searchtext=&orderby=-importance&search=Search&field.status%3Alist=NEW",
                                "delta_url": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + "-changes.html",
                                "changes_file": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + "-changes.csv",
                                "data_file": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + ".data",
                                "delta": "New",
                                "balloon": "New bugs in the past 7 days",
                                "count": "",
                                "description": "New bugs: ",},
                    "incomplete_bugs": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.searchtext=&orderby=-importance&search=Search&field.status%3Alist=INCOMPLETE_WITH_RESPONSE&field.status%3Alist=INCOMPLETE_WITHOUT_RESPONSE&",
                                "delta_url": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + "-changes.html",
                                "changes_file": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + "-changes.csv",
                                "data_file": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + ".data",
                                "delta": "Incomplete",
                                "balloon": "Incomplete bugs in the past 7 days",
                                "count": "",
                                "description": "Incomplete bugs: ",},
                    "confirmed_bugs": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.searchtext=&orderby=-importance&search=Search&field.status%3Alist=CONFIRMED",
                                "delta_url": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + "-changes.html",
                                "changes_file": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + "-changes.csv",
                                "data_file": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + ".data",
                                "delta": "Confirmed",
                                "balloon": "Confirmed bugs in the past 7 days",
                                "count": "",
                                "description": "Confirmed bugs: ",},
                    "triaged_bugs": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.searchtext=&orderby=-importance&search=Search&field.status%3Alist=TRIAGED",
                                "delta_url": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + "-changes.html",
                                "changes_file": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + "-changes.csv",
                                "data_file": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + ".data",
                                "delta": "Triaged",
                                "balloon": "Triaged bugs in the past 7 days",
                                "count": "",
                                "description": "Triaged bugs: ",},
                    "fixed_bugs": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.searchtext=&orderby=-importance&search=Search&field.status%3Alist=FIXRELEASED",
                                "delta_url": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + "-changes.html",
                                "changes_file": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + "-changes.csv",
                                "data_file": "http://people.ubuntu.com/~brian/complete-graphs/" + name + "/" + name + ".data",
                                "delta": "Fix Released",
                                "balloon": "Fix Released bugs in the past 7 days",
                                "count": "",
                                "description": "Fixed bugs: ",},}

        collection = ubuntu.getSourcePackage(name=name)
        get_totals(totals[name], collection)

        xml_pkg_info = xml.createElementNS(None, 'block')
        xml_pkg_info.setAttribute("name", "Package Info")
        xml_pkg.appendChild(xml_pkg_info)

        pkg_info[name] = {"version": {"url": "",
                                "count": "",
                                "baloon": "",
                                "description": "Package Version: ",},
                        }
        get_pkg_info(pkg_info[name], name)
        to_xml(pkg_info[name]["version"], xml_pkg_info)

        xml_bug_info = xml.createElementNS(None, 'block')
        xml_bug_info.setAttribute("name", "Bug Info")
        xml_pkg.appendChild(xml_bug_info)

        xml_totals = xml.createElementNS(None, 'section')
        xml_totals.setAttribute("name", "Totals")
        xml_bug_info.appendChild(xml_totals)

        totals_to_xml(totals[name]["new_bugs"], xml_totals)
        totals_to_xml(totals[name]["incomplete_bugs"], xml_totals)
        totals_to_xml(totals[name]["confirmed_bugs"], xml_totals)
        totals_to_xml(totals[name]["triaged_bugs"], xml_totals)
        totals_to_xml(totals[name]["fixed_bugs"], xml_totals)

        hot_bugs[name] = {"high_bugs": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.importance%3Alist=HIGH",
                        "count": "High",
                        "description": "Bugs with High status: ",},
            "critical_bugs": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.importance%3Alist=CRITICAL",
                            "count": "Critical",
                            "description": "Bugs with Critical status: ",},
            "most_dups": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.searchtext=&orderby=-number_of_duplicates&field.omit_dupes=on",
                    "bug": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/+bug/",
                    "dups": "",
                    "count": "",
                    "title": "",
                    "status": "",
                    "importance": "",
                    "description": "Bug with most duplicates: ",},}

        get_hot_bugs(hot_bugs[name], collection)

        xml_hot_bugs = xml.createElementNS(None, 'section')
        xml_hot_bugs.setAttribute("name", "Hot Bugs")
        xml_bug_info.appendChild(xml_hot_bugs)

        to_xml(hot_bugs[name]["critical_bugs"], xml_hot_bugs)
        to_xml(hot_bugs[name]["high_bugs"], xml_hot_bugs)
        to_xml(hot_bugs[name]["most_dups"], xml_hot_bugs)

        cold_bugs[name] = {"oldest_new": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?orderby=datecreated&search=Search&field.status%3Alist=NEW",
                            "bug": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/+bug/",
                            "status": "New",
                            "importance": "",
                            "title": "",
                            "count": "",
                            "description": "Oldest bug with status New: ",},
            "oldest_confirmed": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?orderby=datecreated&search=Search&field.status%3Alist=CONFIRMED",
                            "bug": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/+bug/",
                            "status": "Confirmed",
                            "importance": "",
                            "title": "",
                            "count": "",
                            "description": "Oldest bug with status Confirmed: ",},
            "oldest_triaged": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?orderby=datecreated&search=Search&field.status%3Alist=TRIAGED",
                            "bug": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/+bug/",
                            "status": "Triaged",
                            "importance": "",
                            "title": "",
                            "count": "",
                            "description": "Oldest bug with status Triaged: ",},}

        get_cold_bugs(cold_bugs[name], collection)

        xml_cold_bugs = xml.createElementNS(None, "section")
        xml_cold_bugs.setAttribute("name", "Cold Bugs")
        xml_bug_info.appendChild(xml_cold_bugs)

        to_xml(cold_bugs[name]["oldest_new"], xml_cold_bugs)
        to_xml(cold_bugs[name]["oldest_confirmed"], xml_cold_bugs)
        to_xml(cold_bugs[name]["oldest_triaged"], xml_cold_bugs)

        upstream[name] = {"fwd_upstream": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.status_upstream=open_upstream&field.status_upstream-empty-marker=1&field.omit_dupes=on",
                                "count": "open_upstream",
                                "description": "Bugs forwarded upstream: ",},
                    "upstream_candidates": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.status_upstream=pending_bugwatch&field.status_upstream-empty-marker=1",
                                "count": "pending_bugwatch",
                                "description": "Upstream candidates: ",},
                        "resolved_upstream":{"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.status_upstream=resolved_upstream&field.omit_dupes=on",
                            "count": "resolved_upstream",
                            "description": "Bugs resolved upstream: "},}

        get_upstream(upstream[name], collection)

        xml_upstream = xml.createElementNS(None, "section")
        xml_upstream.setAttribute("name", "Upstream")
        xml_bug_info.appendChild(xml_upstream)

        to_xml(upstream[name]["fwd_upstream"], xml_upstream)
        to_xml(upstream[name]["upstream_candidates"], xml_upstream)
        to_xml(upstream[name]["resolved_upstream"], xml_upstream)

        regression[name] = {"regression-potential": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.omit_dupes=on&field.tag=regression-potential",
                                                    "count": "regression-potential",
                                                    "balloon": "A bug discovered in the development release that was not present in the stable release.",
                                                    "description": "Regression Potential"},
                            "regression-release": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.omit_dupes=on&field.tag=regression-release",
                                                "count": "regression-release",
                                                "balloon": "A regression in a new stable release.",
                                                "description": "Regression Release"},
                            "regression-update": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name +  "/?field.omit_dupes=on&field.tag=regression-update",
                                                "count": "regression-update",
                                                "balloon": "A regression introduced by an updated package in the stable release.",
                                                "description": "Regression Update"},
                            }

        get_tags(regression[name], collection)
        
        xml_regression = xml.createElementNS(None, "section")
        xml_regression.setAttribute("name", "Regressions")
        xml_bug_info.appendChild(xml_regression)

        to_xml(regression[name]["regression-potential"], xml_regression)
        to_xml(regression[name]["regression-release"], xml_regression)
        to_xml(regression[name]["regression-update"], xml_regression)

        tag[name] = {"bitesize": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.omit_dupes=on&field.tag=bitesize",
                                  "count": "bitesize",
                                  "balloon": "A bug that is easy to fix.",
                                  "description": "Bitesize"},
                    "apport-crash": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.omit_dupes=on&field.tag=apport-crash",
                                  "count": "apport-crash",
                                  "balloon": "A crash reported by apport.",
                                  "description": "Apport Crash"},
                    "apport-bug": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.omit_dupes=on&field.tag=apport-bug",
                                  "count": "apport-bug",
                                  "balloon": "A bug reported via apport.",
                                  "description": "Apport Bug"},
                    }
        get_tags(tag[name], collection)
        
        xml_tag = xml.createElementNS(None, "section")
        xml_tag.setAttribute("name", "Tags")
        xml_bug_info.appendChild(xml_tag)

        to_xml(tag[name]["apport-bug"], xml_tag)
        to_xml(tag[name]["apport-crash"], xml_tag)
        to_xml(tag[name]["bitesize"], xml_tag)

        dev_and_test[name] = {"has_patches": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.status_upstream-empty-marker=1&field.omit_dupes=on&field.has_patch=on",
                                "count": "",
                                "description": "Bugs with patches: ",},
            "sru": {"url": "https://bugs.launchpad.net/ubuntu/+source/" + name + "/?field.subscriber=ubuntu-sru",
                    "count": "",
                    "description": "Bugs in SRU queue: ",},}

        get_has_patches(dev_and_test[name]["has_patches"], collection)
        get_ubuntu_sru(dev_and_test[name]["sru"], collection)

        xml_dev_and_test = xml.createElementNS(None, "block")
        xml_dev_and_test.setAttribute("name", "Development and Testing")
        xml_pkg.appendChild(xml_dev_and_test)

        to_xml(dev_and_test[name]["has_patches"], xml_dev_and_test)
        to_xml(dev_and_test[name]["sru"], xml_dev_and_test)

        additional_refs[name] = {
                                "harvest":
                                    { "url": "http://harvest.ubuntu.com/opportunities/?opp=&opp=list&opp.list=&opp.list=bitesize&opp.list=branches&opp.list=bugpatterns-needed&opp.list=bugs-with-patches.cgi&opp.list=ftbfs&opp.list=jaunty_edosresults&opp.list=karmic&opp.list=maintainer-script&opp.list=packaging&opp.list=patches&opp.list=resolved-upstream&opp.list=reverse_nbs&opp.list=sponsoring&opp.list=SRUs-hardy&opp.list=SRUs-karmic&opp.list=SRUs-lucid&opp.list=SRUs-maverick&opp.list=unmetdeps&opp.list=update-testing-dapper&opp.list=update-testing-hardy&opp.list=update-testing-karmic&opp.list=update-testing-lucid&opp.list=update-testing-maverick&opp.list=upgrade&pkg.set=&pkg=&pkg=name&pkg.name=" + name,
                                    "description": "Harvest",},
                                "debian": 
                                    {"url": "http://packages.qa.debian.org/%s/%s.html" % ( name[0], name ),
                                    "description": "Debian PTS",},
                                }
        xml_additional_refs = xml.createElementNS(None, "block")
        xml_additional_refs.setAttribute("name", "Additional References")
        xml_pkg.appendChild(xml_additional_refs)
        to_xml_link(additional_refs[name]["harvest"], xml_additional_refs)
        to_xml_link(additional_refs[name]["debian"], xml_additional_refs)
        #try:
        #    urllib2.urlopen(additional_refs[name]["debian"]["url"])
        #    to_xml_link(additional_refs[name]["debian"], xml_additional_refs)
        #except HTTPError:
        #    continue

        file = open(name + ".xml", "w")
        try:
            file.write(xml.toprettyxml(encoding="UTF-8"))
        except:
            print "UTF-8 encoding failed"
            file.write(xml.toprettyxml())
        file.close()

file = open(CATEGORY_FILE, "w")
try:
    file.write(xml_doc.toprettyxml(encoding="UTF-8"))
except:
    print "UTF-8 encoding failed"
    file.write(xml_doc.toprettyxml())
file.close()
