#!/usr/bin/env python
##########################################################################
# automatically generate html page which lists bugs tagged "qa-hardy-list"
##########################################################################
import os
import sys
import launchpadbugs.connector as Connector
import commands
import re
import urllib
import getopt
import datetime

def usage():
    print 'python ./support-team-list.py'

TIMESTAMP = commands.getoutput('date -u +"%b %e %G %k:%M %Z"')
TITLE = 'Canonical Support Team Buglist'
HOME = "/home/ogasawara"
PUBLIC_HTML = HOME + "/public_html"
HTML_FILE = PUBLIC_HTML + '/support-team.html'
SUPPORT_TEAM_ARCHIVE = PUBLIC_HTML + "/support-team-archive"
SUPPORT_TEAM_ARCHIVE_URL = "https://chinstrap.canonical.com/~ogasawara/support-team-archive"

FILE = open(HOME + "/.lp", 'r')
email = FILE.readline()
email = email.strip()
password = FILE.readline()
password = password.strip()
#email = raw_input("Enter launchpad username: ")
#password = raw_input("Enter launchpad password: ")

Bug = Connector.ConnectBug('TEXT')
Bug.set_auth({'email': email, 'password': password})

BugList = Connector.ConnectBugList(method='html')
BugList.set_auth({'email': email, 'password': password})

URL = 'https://bugs.launchpad.net/~canonical-support'
buglist = BugList(URL)

html_start = """
<html>
<head>
    <title>%s - %s</title>""" % (TITLE, TIMESTAMP)
html_start += """
    <link defanghtml_rel="made" defanghtml_href="mailto:webmaster@ubuntu.com">
    <link rel="stylesheet" type="text/css" charset="utf-8" media="all" href="http://www.ubuntu.com/htdocs/ubuntuweb/css/common.css">
    <link rel="stylesheet" type="text/css" charset="utf-8" media="screen" href="http://www.ubuntu.com/htdocs/ubuntuweb/css/screen.css">
    <link rel="stylesheet" type="text/css" charset="utf-8" href="buglist.css">
</head>
<body onload=getComments()>

<script type="text/javascript">
var editing  = false;
var global_cell = null;
var public_html = \"%s\";""" % (SUPPORT_TEAM_ARCHIVE_URL)
html_start += """
if (document.getElementById && document.createElement) {
    var button = document.createElement('BUTTON');
    var button_text = document.createTextNode('Save');
    button.appendChild(button_text);
    button.onclick = saveEdit;
}

function readFile(id) {
    var request = new XMLHttpRequest();
    var response;
    var now = new Date().getTime();
    var url = "%s/" + id + ".txt?timestamp=" + now;""" % (SUPPORT_TEAM_ARCHIVE_URL)
html_start += """
    request.open("GET",url,true);
    request.onreadystatechange = function() {
        if (request.readyState != 4) { return; }
        if (request.status==200) {
            response = request.responseText;
            document.getElementById(id).innerHTML=response;
            return response;
        }
        else alert("Error reading " + url);
    }
    request.send(null);
}

function editCell(cell) {
    if (editing) return;

    if (!document.getElementById || !document.createElement) return;
    while (cell.nodeType != 1) {
        cell = cell.parentNode;
    }
    while (cell.nodeName != 'TD' && cell.nodeName != 'HTML') {
        cell = cell.parentNode;
    }
    if (cell.nodeName == 'HTML') return;

    var text = cell.innerHTML;
    var textarea = document.createElement('TEXTAREA');
    var parent = cell.parentNode;
    parent.insertBefore(textarea, cell);
    parent.insertBefore(button, cell);
    global_cell = cell
    parent.removeChild(cell);
    textarea.value = text;
    textarea.focus();
    editing = true;
}

function saveComment(url) {
    var request = new XMLHttpRequest();
    request.open("GET",url,false);
    request.send(null);
}

function saveEdit() {
    var textarea = document.getElementsByTagName('TEXTAREA')[0];
    var parent = textarea.parentNode;
    var url;
    if (textarea.value == "") textarea.value = " ";
    url = public_html + "/comments.py?comment=" + textarea.value + "&bug=" + global_cell.id;
    global_cell.innerHTML = textarea.value;
    saveComment(url);
    parent.insertBefore(global_cell, textarea);
    parent.removeChild(textarea);
    parent.removeChild(document.getElementsByTagName('button')[0]);
    editing = false;
    global_cell = null;
}

"""

javascript = """
function getComments() {
"""
html = " <h3><a href=\"%s\">%s</a> - %s</h3>" % (URL, TITLE, TIMESTAMP)

html += """
<p>
<table border=1>
<tbody>
<tr>
    <th align=center>Bug</defanghtml_hd>
    <th align=center>Summary</th>
    <th align=center>Reported By</th>
    <th align=center>Source Package(s)</th>
    <th align=center>Importance</th>
    <th align=center>Status</th>
    <th align=center>Assignee</th>
    <th align=center>Days Open</th>
    <th align=center>Comments</th>
</tr>
"""

GREEN = '#c4ffc4'
YELLOW = '#fffec4'

for bug in buglist:
    try:
        URL = 'https://bugs.launchpad.net/bugs/%s' % bug.bugnumber
        bug = Bug(url=URL)
        if not bug.private:
            continue
        if "canonical-qa" not in bug.subscribers:
            continue
        packagelist = list(bug.infotable)
        length = len(packagelist)
        count = 0
        html_row_end = ''
        path = SUPPORT_TEAM_ARCHIVE + "/%s.txt" % (bug.bugnumber)

        if not os.path.exists(path):
            FILE = open(path, 'w')
            FILE.close()
            os.chmod(path, 0666)
            comments = ""
        else:
            FILE = open(path, 'r')
            comments = FILE.read()
            FILE.close()
        for i in bug.infotable:
            try:
                if i.status == 'New':
                    color = YELLOW
                elif i.status == "Invalid" or i.status == "Won\'t Fix":
                    continue
                else:
                    color = ''
                html_row = ''
                if count > 0:
                    html_row += "<tr>\n"
                html_row += "\t<td>%s</td>\n" % (i.sourcepackage)
                html_row += "\t<td>%s</td>\n" % i.importance
                html_row += "\t<td bgcolor=\"%s\">%s</td>\n" % (color, i.status)
                html_row += "\t<td>%s</td>\n" % i.assignee
                date = str(i.get_date_created()).split(' ')
                date_split = date[0].split('-')
                today = str(datetime.date.today())
                today_split = today.split('-')
                days = datetime.date(int(today_split[0]), int(today_split[1]), int(today_split[2])) - datetime.date(int(date_split[0]), int(date_split[1]), int(date_split[2]))
                days = str(days).split(' ')
                days_color = ''
                if int(days[0]) > 15:
                    days_color = YELLOW

                html_row += "\t<td bgcolor=\"%s\"><b>%s</b> days (reported: %s)</td>\n" % (days_color, days[0], date[0])
                if count == 0:
                    html_first_row = html_row
                else:
                    html_row += "</tr>\n"
                    html_row_end += html_row

                count += 1
            except:
                print '    Sorry:', sys.exc_type, ':', sys.exc_value
                print '    Omitting package from the list'

        html_row_start = "<tr>\n"
        html_row_start += "\t<td class=\"first\" rowspan=\"%d\"><a href=\"%s\">%d</a></td>\n" % (count, URL, bug.bugnumber)
        html_row_start += "\t<td rowspan=\"%d\">%s</td>\n" % (count, bug.summary)
        html_row_start += "\t<td rowspan=\"%d\">%s</td>\n" % (count, bug.reporter)
        html_row_start += html_first_row
        html_row_start +='\t<td rowspan="%d" id="%s" onclick="return editCell(this)">%s</td>\n</tr>\n' %(count, bug.bugnumber, comments)
        html_row_start += html_row_end
        html += html_row_start
        javascript += '    readFile("%d");\n' % (bug.bugnumber)

    except:
        print '    Sorry:', sys.exc_type, ':', sys.exc_value
        print '    Omitting bug from the list'

html += "</tbody></table></p></body></html>\n"
javascript += "\n}\n</script>\n"
html_start += javascript + html
FILE = open(HTML_FILE, 'w')
FILE.write(html_start)
FILE.close()

