#!/usr/bin/env python
#
# 2011-01-19
# Find out how many bugs a particular package has open and the count
# for each status
# For the importances closed bugs are included to as this is what bugnumbers
# did
# - similar to 'bugnumbers --stats'

from launchpadlib.uris import LPNET_SERVICE_ROOT
from launchpadlib.launchpad import Launchpad
from launchpadlib.credentials import Credentials

import os
import sys
import time

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir, 0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, LPNET_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name,
                                              LPNET_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile, "w", 0600))


ubuntu = launchpad.distributions['ubuntu']
now = int(time.time())
counts = {}
package = sys.argv[1]

target_package = ubuntu.getSourcePackage(name='%s' % package)

open_statuses = ["New", "Incomplete", "Confirmed", "Triaged", "In Progress"]

closed_statuses = ["Fix Committed", "Fix Released", "Invalid", "Won't Fix"]

all_statuses = open_statuses + closed_statuses

importances = ["Undecided", "Wishlist", "Low", "Medium", "High", "Critical"]

open_tasks = target_package.searchTasks(status=open_statuses,
                                               order_by='-datecreated',
                                               omit_duplicates=True)
open_tasks_count = int(open_tasks._wadl_resource.representation['total_size'])

closed_tasks = target_package.searchTasks(status=closed_statuses,
                                               order_by='-datecreated',
                                               omit_duplicates=True)
closed_tasks_count = int(closed_tasks._wadl_resource.representation['total_size'])

counts['total'] = open_tasks_count + closed_tasks_count

counts['date'] = now

if open_tasks_count != 0:
    for open_status in open_statuses:
        status_count = int(target_package.searchTasks(status='%s' % open_status, omit_duplicates=True)._wadl_resource.representation['total_size'])
        status_key = open_status.replace(" ", "").lower()
        counts[status_key] = status_count
elif open_tasks_count == 0:
    for open_status in open_statuses:
        status_key = open_status.replace(" ", "").lower()
        counts[status_key] = 0

for closed_status in closed_statuses:
    status_count = int(target_package.searchTasks(status='%s' % closed_status, omit_duplicates=True)._wadl_resource.representation['total_size'])
    status_key = closed_status.replace(" ","").replace("'","").lower()
    counts[status_key] = status_count
for importance in importances:
    importance_count = int(target_package.searchTasks(status=all_statuses, importance='%s' % importance, omit_duplicates=True)._wadl_resource.representation['total_size'])
    counts[importance.lower()] = importance_count

ordering = ['date', 'total', 'new', 'incomplete', 'confirmed', 'triaged',
            'inprogress', 'fixcommitted', 'fixreleased', 'invalid', 'wontfix',
            'undecided', 'wishlist', 'medium', 'high', 'critical']

csv_count = []
for item in ordering:
    csv_count.append(str(counts[item]))

print ', '.join(csv_count)
