#!/usr/bin/env python
# 
# 2009-07-23 
# Find out how many bugs a particular package has open and the count
# for each status / importance - similar to 'bugnumbers --stats'
#
# Replace STAGING_SERVICE_ROOT with EDGE_SERVICE_ROOT as needed

from launchpadlib.uris import LPNET_SERVICE_ROOT
from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT, STAGING_SERVICE_ROOT
from launchpadlib.credentials import Credentials

import simplejson as json
import os
import sys
import time

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir, 0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, LPNET_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, LPNET_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile, "w", 0600))


def load_existing_stats(data_filename):
    try:
        data_file = open(data_filename, 'r')
    except IOError:
        return
    json_data = json.load(data_file)
    data_file.close()

    for record in json_data['records']:
        records.append(record)

ubuntu = launchpad.distributions['ubuntu']
now = int(time.time())
records = []
counts = {}
package = sys.argv[1]
package_stats_filename = "%s/%s-bug-stats.json" % (package, package)

load_existing_stats(package_stats_filename)

target_package = ubuntu.getSourcePackage(name='%s' % package)

open_statuses = ["New", "Incomplete", "Confirmed", "Triaged", "In Progress"]

closed_statuses = ["Fix Committed", "Fix Released", "Invalid", "Won't Fix"]

importances = ["Undecided", "Wishlist", "Low", "Medium", "High", "Critical"]

open_tasks = target_package.searchTasks(status=open_statuses,
                                               order_by='-datecreated',
                                               omit_duplicates=True)
open_tasks_count = int(open_tasks._wadl_resource.representation['total_size'])


counts['date'] = now

if open_tasks_count != 0:
    for open_status in open_statuses:
        status_count = int(target_package.searchTasks(status='%s' % open_status, omit_duplicates=True)._wadl_resource.representation['total_size'])
        status_key = open_status.replace(" ", "").lower()
        counts[status_key] = status_count
elif open_tasks_count == 0:
    for open_status in open_statuses:
        status_key = open_status.replace(" ", "").lower()
        counts[status_key] = 0

for closed_status in closed_statuses:
    status_count = int(target_package.searchTasks(status='%s' % closed_status, omit_duplicates=True)._wadl_resource.representation['total_size'])
    status_key = closed_status.replace(" ","").replace("'","").lower()
    counts[status_key] = status_count
for importance in importances:
    importance_count = int(target_package.searchTasks(importance='%s' % importance, omit_duplicates=True)._wadl_resource.representation['total_size'])
    counts[importance.lower()] = importance_count

records.append(counts)

report = {
          'keys': ['date', 'new', 'incomplete', 'confirmed', 'triaged',
                   'inprogress', 'fixcommitted', 'fixreleased', 'invalid',
                   'wontfix', 'undecided', 'wishlist', 'low', 'medium',
                   'high', 'critical'],
          'records': records
         }

package_stats_file = open(package_stats_filename, 'w')
package_stats_file.write(json.dumps(report))
package_stats_file.close()
