#!/usr/bin/python

from subprocess import Popen, PIPE
import simplejson as json
import sys
import time


def load_existing_stats(ps_stats_filename):
    try:
        ps_stats_file = open(ps_stats_filename, 'r')
    except IOError:
        return
    json_data = json.load(ps_stats_file)
    ps_stats_file.close()

    for record in json_data['records']:
        records.append(record)

def get_latest_ps_pkg_stats(ps_pkg_file):
    ps_new = 0
    ps_incomplete = 0
    ps_confirmed = 0
    ps_triaged = 0
    ps_inprogress = 0
    ps_fixcommitted = 0
    ps_fixreleased = 0
    ps_invalid = 0
    ps_wontfix = 0
    ps_undecided = 0
    ps_wishlist = 0
    ps_low = 0
    ps_medium = 0
    ps_high = 0
    ps_critical = 0
    for package in ps_pkg_file:
        package = package.strip('\n')
        pkg_datafile = open("%s/%s-bug-stats.json" % (package, package), 'r')
        pkg_data = json.load(pkg_datafile)
        pkg_datafile.close()

        ps_new += int(pkg_data['records'][-1]['new'])
        ps_incomplete += int(pkg_data['records'][-1]['incomplete'])
        ps_confirmed += int(pkg_data['records'][-1]['confirmed'])
        ps_triaged += int(pkg_data['records'][-1]['triaged'])
        ps_inprogress += int(pkg_data['records'][-1]['inprogress'])
        ps_fixcommitted += int(pkg_data['records'][-1]['fixcommitted'])
        ps_fixreleased += int(pkg_data['records'][-1]['fixreleased'])
        ps_invalid += int(pkg_data['records'][-1]['invalid'])
        ps_wontfix += int(pkg_data['records'][-1]['wontfix'])
        ps_undecided += int(pkg_data['records'][-1]['undecided'])
        ps_wishlist += int(pkg_data['records'][-1]['wishlist'])
        ps_low += int(pkg_data['records'][-1]['low'])
        ps_medium += int(pkg_data['records'][-1]['medium'])
        ps_high += int(pkg_data['records'][-1]['high'])
        ps_critical += int(pkg_data['records'][-1]['critical'])

    ps_pkg_file.close()
    return ps_new, ps_incomplete, ps_confirmed, ps_triaged, ps_inprogress, \
           ps_fixcommitted, ps_fixreleased, ps_invalid, ps_wontfix, \
           ps_undecided, ps_wishlist, ps_low, ps_medium, ps_high, ps_critical

ps_pkg_filename = sys.argv[1]
ps_pkg_file = open(ps_pkg_filename, 'r')
ps_stats_filename = "%s-bug-stats.json" % \
                     (ps_pkg_filename.replace('-packages.txt',''))

records = []

load_existing_stats(ps_stats_filename)

ps_new, ps_incomplete, ps_confirmed, ps_triaged, ps_inprogress, ps_fixcommitted, ps_fixreleased, ps_invalid, ps_wontfix, ps_undecided, ps_wishlist, ps_low, ps_medium, ps_high, ps_critical = get_latest_ps_pkg_stats(ps_pkg_file)

now = int(time.time())

records.append({
    'date': now,
    'new': ps_new,
    'incomplete': ps_incomplete,
    'confirmed': ps_confirmed,
    'triaged': ps_triaged,
    'inprogress': ps_inprogress,
    'fixcommitted': ps_fixcommitted,
    'fixreleased': ps_fixreleased,
    'invalid': ps_invalid,
    'wontfix': ps_wontfix,
    'undecided': ps_undecided,
    'wishlist': ps_wishlist,
    'low': ps_low,
    'medium': ps_medium,
    'high': ps_high,
    'critical': ps_critical
})

report = {
          'keys': ['date', 'new', 'incomplete', 'confirmed', 'triaged',
                   'inprogress', 'fixcommitted', 'fixreleased', 'invalid',
                   'wontfix', 'undecided', 'wishlist', 'low', 'medium', 
                   'high', 'critical'],
          'records': records}

ps_stats_file = open(ps_stats_filename, 'w')
ps_stats_file.write(json.dumps(report))
ps_stats_file.close()
