#!/usr/bin/python
# pass the script potential, release or what have you
#
# reads a bug stat csv file and present stats, on open bugs
# for the release cycle to be used in conjuction with
# regression-release-comparison.py 
#
# N.B. - The Fix Released count is not release specific as a
# regression-potential bug tagged in Jaunty which is Fixed then doesn't get
# untagged just because Karmic came out

import datetime
import csv
import urllib
import sys

try:
    stat = sys.argv[1]
except IndexError:
    print "Must provide a tag: regression-potential, regression-release or regression-updates"
    sys.exit(1)

def present_results(date, stats):
    open = 0
    data = []
    stati = ['New', 'Incomplete', 'Confirmed', 'Triaged', 'In Progress', 'Fix Committed', 'Fix Released']
    data.append('%s' % date)
    # don't include fix released ones in the open count
    for k,v in stats.items():
        if k != 'Fix Released':
            open += int(v)
    for status in stati:
        data.append('%s' % stats[status])
    data.insert(1, '%s' % open)
    return(', '.join(data))

releases = [ 'intrepid', 'jaunty', 'karmic', 'lucid', 'maverick' ]

# could do this via launchpadlib and date_created / datereleased for distro series
release_dates = { 'intrepid': {'start':datetime.date(2008,04,28),
                               'end':datetime.date(2008,10,30)},
                  'jaunty':   {'start':datetime.date(2008,10,31),
                               'end':datetime.date(2009,04,23)},
                  'karmic':   {'start':datetime.date(2009,04,24),
                               'end':datetime.date(2009,10,29)},
                  'lucid':    {'start':datetime.date(2009,10,30),
                               'end':datetime.date(2010,04,29)},
                  'maverick': {'start':datetime.date(2010,05,01),
                               'end':datetime.date(2010,10,10)},
                }

datafile = "http://people.canonical.com/~brian/complete-graphs/%s/%s.data" % (stat, stat)

try:
    reader = csv.reader(urllib.urlopen(datafile))
except IOError:
    print '''data file doesn't exist.'''

stats = {}

for release in releases:
    stats[release] = {}

for row in reader:
    if row[0].startswith('#'):
        continue
    data_date = datetime.date.fromtimestamp(int(row[0])) 
    for release in releases:
        if data_date >= release_dates[release]['start'] and data_date <= release_dates[release]['end']:
            day_number = ( data_date - release_dates[release]['start'] ).days
            # not using append only want the last stat for that day
            stats[release][day_number] =  { 'New':row[2].strip(), 'Incomplete':row[3].strip(), 'Confirmed':row[4].strip(), 'Triaged':row[5].strip(), 'In Progress':row[6].strip(), 'Fix Committed':row[7].strip(), 'Fix Released':row[8].strip() }

for release in releases:
    day_list = stats[release].keys()
    day_list.sort()

    results_file = open('%s-%s.data' % (release, stat), 'w')

    # add in a header just in case someone is reading the csv
    results_file.write('# day number in cycle, open, new, incomplete, confirmed, triaged, in progress, fix committed, fix released\n')

    for day in day_list:
        results_file.write(present_results(day, stats[release][day]) + "\n")

    results_file.close
