#!/usr/bin/env python
# 
# 2009-07-23 
# Find out how many bugs a particular package has open and the count
# for each status / importance - similar to 'bugnumbers --stats'
#
# Replace STAGING_SERVICE_ROOT with EDGE_SERVICE_ROOT as needed

from launchpadlib.uris import LPNET_SERVICE_ROOT
from launchpadlib.launchpad import Launchpad
from launchpadlib.credentials import Credentials

import simplejson as json
import os
import sys
import time

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir, 0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, LPNET_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, LPNET_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile, "w", 0600))


def load_existing_stats(data_filename):
    try:
        data_file = open(data_filename, 'r')
    except IOError:
        return
    json_data = json.load(data_file)
    data_file.close()

    for record in json_data['records']:
        records.append(record)


def find_active_releases():
    releases = []
    for series in ubuntu.series:
        if series.active:
            releases.append(series)
        elif not series.active:
            continue
    return releases


def find_targetted(release):
    targetted_bugs = [task.bug for task in \
                       release.searchTasks(omit_targeted=False)]

    return targetted_bugs

ubuntu = launchpad.distributions['ubuntu']
releases = find_active_releases()
for release in releases:
    now = int(time.time())
    records = []
    counts = {}
    release_stats_filename = "%s/%s-bug-stats.json" % (release.name, release.name)

    load_existing_stats(release_stats_filename)

    targetted_bugs = find_targetted(release)

    open_statuses = ["New", "Incomplete", "Confirmed", "Triaged",
                     "In Progress", "Fix Committed"]

    closed_statuses = ["Fix Released", "Invalid", "Won't Fix"]

    importances = ["Undecided", "Wishlist", "Low", "Medium", "High",
                   "Critical"]

    open_tasks = release.searchTasks(status=open_statuses,
                                     omit_duplicates=True,
                                     omit_targeted=False)
    open_tasks_count = int(open_tasks._wadl_resource.representation['total_size'])
    counts['open'] = open_tasks_count

    counts['date'] = now

    if open_tasks_count != 0:
        for open_status in open_statuses:
            status_count = int(release.searchTasks(status='%s' % open_status, omit_duplicates=True, omit_targeted=False)._wadl_resource.representation['total_size'])
            status_key = open_status.replace(" ", "").lower()
            counts[status_key] = status_count
    elif open_tasks_count == 0:
        for open_status in open_statuses:
            status_key = open_status.replace(" ", "").lower()
            counts[status_key] = 0

    for closed_status in closed_statuses:
        status_count = int(release.searchTasks(status='%s' % closed_status, omit_duplicates=True, omit_targeted=False)._wadl_resource.representation['total_size'])
        status_key = closed_status.replace(" ", "").replace("'", "").lower()
        counts[status_key] = status_count
    for importance in importances:
        importance_count = int(release.searchTasks(importance='%s' % importance, omit_duplicates=True, omit_targeted=False)._wadl_resource.representation['total_size'])
        counts[importance.lower()] = importance_count

    records.append(counts)

    report = {
              'keys': ['date', 'new', 'incomplete', 'confirmed', 'triaged',
                       'inprogress', 'fixcommitted', 'fixreleased', 'invalid',
                       'wontfix', 'undecided', 'wishlist', 'low', 'medium',
                       'high', 'critical', 'open'],
              'records': records
             }

    release_stats_file = open(release_stats_filename, 'w')
    release_stats_file.write(json.dumps(report, indent=4))
    release_stats_file.close()
