#!/usr/bin/python
#

from launchpadlib.uris import LPNET_SERVICE_ROOT
from launchpadlib.launchpad import Launchpad
from launchpadlib.credentials import Credentials

import os
import simplejson as json
import sys

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir, 0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, LPNET_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name,
                                              LPNET_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile, "w", 0600))


def load_existing_stats(release_stats_filename):
    try:
        release_stats_file = open(release_stats_filename, 'r')
    except IOError:
        return None
    json_data = json.load(release_stats_file)
    release_stats_file.close()

    return json_data['records'][-29], json_data['records'][-1]


def write_table_header():
    header = "||"
    columns = ["'''Release'''", "'''Bugs Open'''", "'''Critical'''",
               "'''High'''", "'''New'''"]
    for column in columns:
        header += " "
        header += column
        header += " ||"
    header += "\n"
    moin_file.write(header)


def write_table_row(old_data, data):
    codename = release_name.title()
    base_url = 'http://bugs.launchpad.net/ubuntu/%s/+bugs' % codename.lower()
    keys = ['open', 'critical', 'high', 'new']
    row = "||"
    row += " %s ||" % codename
    for key in keys:
        row += " "
        row += "[[%s" % base_url
        if key == 'critical' or key == 'high':
            row += '?field.importance=%s|' % key.title()
        elif key == 'new':
            row += '?field.status=%s|' % key.title()
        else:
            row += '|'
        row += "%s" % data[key]
        difference = str(data[key] - old_data[key])
        if difference.startswith('-') or difference == '0':
            row += " (%s)" % difference
        else:
            row += " (+%s)" % difference
        row += "]]"
        row += " ||"
    row += "\n"
    moin_file.write(row)


def find_active_releases():
    releases = []
    for series in ubuntu.series:
        if series.status == 'Active Development':
            continue
        if series.active:
            releases.append(series)
        elif not series.active:
            continue
    return releases


ubuntu = launchpad.distributions['ubuntu']
releases = find_active_releases()


release_stats_moinname = 'release-bug-stats.moin'
moin_file = open(release_stats_moinname, 'w')
write_table_header()
for release in releases:
    release_name = release.name
    records = []
    release_stats_filename = "%s/%s-bug-stats.json" % (release_name,
                                                       release_name)
    #release_stats_file = open(release_stats_filename, 'r')
    last_week_data, current_data = load_existing_stats(release_stats_filename)
    if current_data:
        write_table_row(last_week_data, current_data)
    else:
        continue

moin_file.close()
