#!/usr/bin/env python
#
# 2010-04-19
# Find information about quantity of bugs with patches and the queues they are
# in

from launchpadlib.launchpad import Launchpad, EDGE_SERVICE_ROOT, STAGING_SERVICE_ROOT
from launchpadlib.errors import HTTPError
from launchpadlib.credentials import Credentials

import os, sys, time

def count_patches(team):
    open_stati = ['New', 'Incomplete', 'Confirmed', 'Triaged', 'In Progress']
    bug_tasks = ubuntu.searchTasks(has_patch=True,status=open_stati,bug_subscriber=team)
    patches_count = int(bug_tasks._wadl_resource.representation['total_size'])
    return patches_count

cachedir = os.path.expanduser("~/.launchpadlib/cache/")

if not os.path.exists(cachedir):
    os.makedirs(cachedir,0700)

script_name = sys.argv[0].split("/")[-1].split('.')[0]

credfile = os.path.expanduser('~/.launchpadlib/%s.cred' % script_name)

try:
    credentials = Credentials()
    credentials.load(open(credfile))
    launchpad = Launchpad(credentials, EDGE_SERVICE_ROOT, cachedir)
except:
    launchpad = Launchpad.get_token_and_login(script_name, EDGE_SERVICE_ROOT, cachedir)
    launchpad.credentials.save(open(credfile,"w",0600))

datafile = open('stats-bugs-with-patches.csv', 'a')

ubuntu = launchpad.distributions['ubuntu']

ran = int(time.time())

umr = launchpad.people['ubuntu-reviewers']
ums = launchpad.people['ubuntu-main-sponsors']
# 2010-10-25 XXX - team no longer exists so commenting out
#uus = launchpad.people['ubuntu-universe-sponsors']
us = launchpad.people['ubuntu-sponsors']
patch_workflow_tags = [
    'patch-needswork',
    'patch-forwarded-upstream',
    'patch-forwarded-debian',
    'patch-accepted-upstream',
    'patch-accepted-debian',
    'patch-rejected-upstream',
    'patch-rejected-debian',
    'patch-rejected'
    ]

total_patches = ubuntu.searchTasks(has_patch=True,status=['New','Incomplete','Confirmed','Triaged','In Progress'])
total_patches_count = int(total_patches._wadl_resource.representation['total_size'])

fixed_patches = ubuntu.searchTasks(has_patch=True,status=['Fix Released'])
fixed_patches_count = int(fixed_patches._wadl_resource.representation['total_size'])

already_reviewed_patches = ubuntu.searchTasks(
    has_patch = True,
    status = ['New','Incomplete','Confirmed','Triaged','In Progress'],
    tags = patch_workflow_tags,
    tags_combinator='Any'
    )

# Subtract these from the to_review_patches_count as they have already been reviewed
# also saving the number just in case
already_reviewed_patches_count = int(already_reviewed_patches._wadl_resource.representation['total_size'])

# 2010-06-10 - new query for nigelb
cleansweep_progress = ubuntu.searchTasks(
    has_patch = True,
    status = ['New','Incomplete','Confirmed','Triaged','In Progress','Fix Committed'],
    # want the negated search here so add - in front of every tag
    tags = ['-%s' % tag for tag in patch_workflow_tags],
    tags_combinator = 'All',
    bug_subscriber = umr
    )

cleansweep_progress_count = int(cleansweep_progress._wadl_resource.representation['total_size'])

umr_patches_count = count_patches(umr)
ums_patches_count = count_patches(ums)
# 2010-10-25 - XXX setting count to zero as team no longer exists
uus_patches_count = 0
us_patches_count = count_patches(us)

to_review_patches_count = (total_patches_count - ums_patches_count -
                           uus_patches_count - us_patches_count -
                           already_reviewed_patches_count )

datafile.write('%s, %s, %s, %s, %s, %s, %s, %s, %s, %s\n' % (ran,
    total_patches_count, to_review_patches_count, umr_patches_count,
    ums_patches_count, uus_patches_count, fixed_patches_count, us_patches_count,
    already_reviewed_patches_count, cleansweep_progress_count)
    )
#print('%s, %s, %s, %s, %s, %s, %s, %s, %s\n' % (ran, total_patches_count, to_review_patches_count, umr_patches_count, ums_patches_count, uus_patches_count, fixed_patches_count, us_patches_count, already_reviewed_patches_count))

datafile.close()
