#!/usr/bin/python
# Author: Brian Murray <brian@canonical.com>
# Copyright (C) 2011 Canonical, Ltd.
# License: GPLv3

from datetime import date
from time import time
import simplejson as json
import sys

import lpl_common


def load_existing_stats(data_filename):
    try:
        data_file = open(data_filename, 'r')
    except IOError:
        return
    json_data = json.load(data_file)
    data_file.close()

    for record in json_data['records']:
        records.append(record)

lp = lpl_common.connect(use_edge=False, version='devel')


if __name__ == '__main__':
    data = {}
    records = []
    ages = []
    data['date'] = int(time())

    ubuntu = lp.distributions['ubuntu']
    team_name = sys.argv[1]
    team = lp.people[team_name]
    team_stats_filename = "%s-assigned-bug-stats.json" % (team_name)

    load_existing_stats(team_stats_filename)
    assigned_tasks = [task for task in ubuntu.searchTasks(assignee=team)]
    data['quantity'] = len(assigned_tasks)

    today = date.today()
    for task in assigned_tasks:
        day_assigned = task.date_assigned.date()
        age = (today - day_assigned)
        ages.append(age.days)

    if data['quantity'] != 0:
        data['avg-age'] = sum(ages, 0) / len(ages)
        data['oldest'] = max(ages)
    else:
        data['avg-age'] = 0
        data['oldest'] = 0


    records.append(data)

    report = {'keys': ['date', 'quantity', 'avg-age'],
              'records': records}

    team_stats_file = open(team_stats_filename, 'w')
    team_stats_file.write(json.dumps(report, indent=4))
